#!/bin/sh
#
# NAG Fortran Compiler installation script
#
# Can install to user-specified or system-specific default locations
#
# Release 5.1: 1.5, 07/04/02
#
# Release 5.2: CDINSTALL.sh 35 2010-01-25 16:47:01Z ianh
#
# Release 5.3: $Id: CDINSTALL.sh 2165 2012-05-16 14:06:16Z ianh $
#
remove=rm
removedir=rmdir
removetree='rm -rf'
whoami=whoami
chown='chown -fhR'
chgrp='chgrp -fhR'
# 
# INSTALL settings
#
#move='mv -f'
#maybe_remove=rm
#maybe_removetree='rm -rf'
#installer='INSTALL.sh'
#
# CDINSTALL settings
#
move='cp'
maybe_remove=:
maybe_removetree=:
installer='CDINSTALL.sh'
#
if [ ! -t 0 ]; then
	echo "Do 'sh $installer' not 'sh<$installer'"
	exit 1
fi
if [ ! -f $installer -o ! -d bin -o ! -d lib -o ! -d man -o ! -x bin/nagfor ]; then
	newd=`dirname $0`
	if [ -f $newd/$installer -a -d $newd/lib -a -x $newd/bin/nagfor ]; then
		echo Changing working directory to $newd...
		cd $newd
	else
		echo "Error: Cannot find NAG Fortran Compiler distribution directory"
		echo
		echo "Please change your working directory to the relevant"
		echo "NAG_Fortran-* distribution directory and try again."
		exit 1
	fi
fi
libdir=/usr/local/lib/NAG_Fortran
bindir=/usr/local/bin
man1dir=/usr/local/man/man1
cat1dir=/usr/local/man/cat1
man1suffix=1
man3dir=/usr/local/man/man3
cat3dir=/usr/local/man/cat3
man3suffix=3
defaultlib='YES'
defaultman='YES'
preforman='NO'
if [ -x /usr/bin/uname -o -x /bin/uname ]; then
	if [ `uname` = "SunOS" ]; then
	        if [ $installer = "CDINSTALL" ]; then
	                move='cp -p'
	        fi
	        if expr `uname -r` \>= 5.0 >/dev/null; then
			whoami=/usr/ucb/whoami
			libdir=/opt/NAG_Fortran/lib
			bindir=/opt/NAG_Fortran/bin
		fi
	elif [ `uname` = "Darwin" ]; then
	        bindir=/usr/bin
	elif [ `uname` = "HP-UX" -o `uname` = "Linux" ]; then
		chown='chown -R'
		chgrp='chgrp -R'
	fi
fi
whoami=`$whoami`
echo
echo "Welcome to the NAG Fortran Compiler Installation script!"
echo
# Work out how to write prompts without newline
(echo "hi there\c" ; echo " ") >/tmp/.echotmp
if grep c /tmp/.echotmp >/dev/null 2>&1 ; then
    n='-n'
    c=''
else
    n=''
    c='\c'
fi
$remove /tmp/.echotmp
#
#
needed_dirs=""
echo "Default answers to the questions are in square brackets [] -"
echo "just press the RETURN key to accept the default."
echo
licence_agreement_file=lic_agr.txt
if [ -f ${licence_agreement_file} ]; then
    more ${licence_agreement_file}
else
    echo "Cannot find licence agreement file ${licence_agreement_file}."
    echo "NAG Fortran Compiler not installed."
    exit 1
fi
echo $n "Do you accept this licence agreement [Y]?" $c
read ans
if [ -n "$ans" -a "$ans" != "y" -a "$ans" != "Y" -a "$ans" != "Yes" -a "$ans" != "YES" -a "$ans" != "yes" ]; then
    echo "Installation declined. NAG Fortran Compiler not installed."
    exit 1
fi
echo
echo $n "Install compiler binaries to where [$bindir]?" $c
read ans
if [ -n "$ans" ]; then
    if [ "$ans" != $bindir -a `basename "$ans"` = "bin" ]; then
        newbase=`dirname "$ans"`
        if [ `uname` = "SunOS" ]; then
           libdir=$newbase/lib
        else
           libdir=$newbase/lib/NAG_Fortran
        fi
        man1dir=$newbase/man/man1
        man3dir=$newbase/man/man3
    fi
    bindir=$ans
fi
if [ -f $bindir ]; then
	echo $bindir is a normal file, not a directory
	exit 1
fi
needed_dirs="$needed_dirs $bindir"
#
echo
echo $n "Install compiler library files to where [$libdir]?" $c
read ans
if [ -n "$ans" ]; then
	if [ $libdir != $ans ]; then
		defaultlib='NO'
	fi
	libdir=$ans
fi
if [ -f $libdir ]; then
	echo $libdir is a normal file, not a directory
	exit 1
elif [ $bindir = $libdir ]; then
	echo Library directory must be separate from binary directory.
	exit 1
fi
needed_dirs="$needed_dirs $libdir"
#
if [ -f man/nagfor.z ]; then
	echo
	echo $n "Install pre-formatted man pages [Y]?" $c
	read ans
	if [ -z "$ans" -o "$ans" = "y" -o "$ans" = "Y" -o "$ans" = "Yes" -o "$ans" = "YES" -o "$ans" = "yes" ]; then
	        preforman='YES'
		echo
		echo $n "Install pre-formatted compiler man pages to which directory [$cat1dir]?" $c
		read ans
		if [ -n "$ans" ]; then
			cat1dir=$ans
		fi
		if [ -f $cat1dir ]; then
			echo $cat1dir is a normal file, not a directory
			exit 1
		fi
		needed_dirs="$needed_dirs $cat1dir"
		echo
		echo $n "Install pre-formatted module man pages to which directory [$cat3dir]?" $c
		read ans
		if [ -n "$ans" ]; then
			cat3dir=$ans
		fi
		if [ -f "$cat3dir" ]; then
			echo $cat3dir is a normal file, not a directory
			exit 1
		fi
		needed_dirs="$needed_dirs $cat3dir"
		echo $n "Install unformatted man pages as well [N]?" $c
		read ans
		if [ -z "$ans" -o "$ans" != "y" -a "$ans" != "Y" -a "$ans" != "Yes" -a "$ans" != "YES" -a "$ans" != "yes" ]; then
			defaultman='NO'
		fi
	fi
fi
#
if [ $defaultman = "YES" ]; then
	echo
	echo $n "Install compiler man page to which directory [$man1dir]?" $c
	read ans
	if [ -n "$ans" ]; then
		man1dir=$ans
	fi
	if [ -f $man1dir ]; then
		echo $man1dir is a normal file, not a directory
		exit 1
	fi
	needed_dirs="$needed_dirs $man1dir"
#
	echo
	echo $n "Suffix for compiler man page [$man1suffix] (i.e. nagfor.$man1suffix)?" $c
	read ans
	if [ -n "$ans" ]; then
		man1suffix=$ans
	fi
#
	echo
	echo $n "Install module man pages to which directory [$man3dir]?" $c
	read ans
	if [ -n "$ans" ]; then
		man3dir=$ans
	fi
	if [ -f $man3dir ]; then
		echo $man3dir is a normal file, not a directory
		exit 1
	fi
	needed_dirs="$needed_dirs $man3dir"
#
	echo
	echo $n "Suffix for module man pages [$man3suffix] (i.e. f90_gc.$man3suffix)?" $c
	read ans
	if [ -n "$ans" ]; then
		man3suffix=$ans
	fi
fi
#
if [ -x $libdir/forcomp ]; then
	echo $n "Old installation found - remove contents of $libdir [Y]?" $c
	read ans
	if [ -n "$ans" -a "$ans" != "y" -a "$ans" != "Y" -a "$ans" != "Yes" -a "$ans" != "YES" -a "$ans" != "yes" ]; then
		echo $libdir contents NOT removed - attempting to overwrite...
	else
	    if [ -x $libdir/UNINSTALL_NAGFOR ]; then
		$libdir/UNINSTALL_NAGFOR
	        echo $libdir contents removed using UNINSTALL_NAGFOR script
	    else
	        $remove -rf $libdir
	        echo $libdir contents removed
            fi
	fi
fi
#
if [ $whoami = "root" ]; then
	echo $n "Set ownership of files to [root]?" $c
	read ans
	if [ -n "$ans" ]; then
		owner=$ans
	else
		owner=root
	fi
	echo $n "Set group of files to [bin]?" $c
	read ans
	if [ -n "$ans" ]; then
		group=$ans
	else
		group=bin
	fi
fi
#
# Start by creating directories - must be done after UNINSTALL_NAGFOR called
nagdirs=""
for i in $needed_dirs; do    
    while [ ! -d $i ]; do
	create=$i
	parent=`dirname $create`
	while [ ! -d $parent -a -n "$parent" ]; do
	    create=$parent
	    parent=`dirname $create`
	done
	if mkdir $create; then
	    echo Created $create
	    nagdirs="$create $nagdirs"
	else
	    echo Cannot create $create
	    exit 1
	fi
    done
done
echo $n Installing $c
#
# Make sure we can create this
$remove -f $bindir/nagfor
uninstall="$libdir/UNINSTALL_NAGFOR"
echo "#!/bin/sh" > $uninstall
echo "files=\"$bindir/nagfor \\" >> $uninstall
$remove -f $bindir/nagfmcheck
$move bin/nagfmcheck $bindir
echo "	$bindir/nagfmcheck \\" >> $uninstall
if [ "$defaultlib" = "YES" ]; then
	$move bin/nagfor $bindir
else
	$move bin/nagfor $libdir
	echo "#!/bin/sh" >$bindir/nagfor
	echo $libdir/nagfor '"$@"' -Qpath $libdir >>$bindir/nagfor
	chmod 755 $bindir/nagfor
	echo "  $libdir/nagfor \\" >> $uninstall
fi
if [ -x bin/dbx90 ]; then
	$remove -f $bindir/dbx90
        if [ "$defaultlib" = "YES" ]; then
	    $move bin/dbx90 $bindir
        else
	    $move bin/dbx90 $libdir
	    echo "#!/bin/sh" >$bindir/dbx90
	    echo $libdir/dbx90 -Qpath $libdir '"$@"' >>$bindir/dbx90
	    chmod 755 $bindir/dbx90
	    echo "  $libdir/dbx90 \\" >> $uninstall
        fi
	echo "  $bindir/dbx90 \\" >> $uninstall
fi
if [ -x lib/upsf95 ]; then
	$remove -f $bindir/upsf95
	echo "#!/bin/sh" >$bindir/upsf95
	echo XAPPLRESDIR=$libdir >>$bindir/upsf95
	echo export XAPPLRESDIR >>$bindir/upsf95
	echo exec $libdir/upsf95 '"$@"' >>$bindir/upsf95
	chmod 755 $bindir/upsf95
	echo "  $bindir/upsf95 \\" >> $uninstall
fi
$maybe_removetree bin
echo $n . $c
$move lib/forcomp $libdir
echo $n . $c
tar cf - doc html | (cd $libdir ; tar xf -)
cd lib
tar cf - . | (cd $libdir ; tar xf -)
mods="f90_kind.mod f90_iostat.mod f90_gc.mod f90_unix_errno.mod \
      f90_unix_env.mod f90_unix_dir.mod f90_unix_dirent.mod \
      f90_unix_file.mod f90_unix_io.mod f90_unix_proc.mod \
      f90_unix.mod f90_preconn_io.mod ieee_exceptions.mod \
      ieee_arithmetic.mod ieee_features.mod f90_stat.mod \
      iso_fortran_env.mod iso_c_binding.mod omp_lib.mod"
other="debug.o dope.h f90_iostat.f90 f90_kind.f90 f90_util.f90 \
       nagfortran.h gc.o hpf.h quickfit.o safefit.o thsafe.o gc_static.o \
       so_locations libf53.so.1.0 libf53.so.1 libf53.so libf53.a libf53.dylib \
       libunshared.a sparc_names.h ddbasic.h libfm.h f90_stat.f90 \
       khostid klcheck kldctl kdongle maudoff.o maudon.o \
       paraon.o paraoff.o tpcfit.o tpckludge.o traceaddr.o tracebasic.o \
       traceline.o tracenoaddr.o tracenoline.o tracenosize.o traceoff.o \
       traceon.o tracesize.o Ups upsf95 ups32 ups64 nagfmcheck.f90 \
       iso_fortran_env.f90 f90_init.o f90_initu.o underoff.o underon.o \
       omplib32.a omplib64.a fcalls.exe fcalls64.exe polish.exe fditool.exe"
for i in $other $mods; do
    if [ -f $i ]; then
        echo "  $libdir/$i \\" >> $uninstall
    fi
done
for j in bytes n32 o32 x86_32 x86_64f uv uvbytes; do
    if [ -d $libdir/$j ]; then
        for i in $mods $other; do
            if [ -f $libdir/$j/$i ]; then
                echo "  $libdir/$j/$i \\" >> $uninstall
            fi
        done
	nagdirs="$libdir/$j $nagdirs"
    fi
done
if [ -d x86_32 ]; then
    echo "  $libdir/x86_32/uv \\" >> $uninstall
    echo "  $libdir/x86_32/uvbytes \\" >> $uninstall
    for i in $mods $other; do
        if [ -f $libdir/x86_32/bytes/$i ]; then
            echo "  $libdir/x86_32/bytes/$i \\" >> $uninstall
        fi
    done
    nagdirs="$libdir/x86_32/bytes $nagdirs"
fi
if [ -d x86_64f ]; then
    echo "  $libdir/x86_64f/bytes \\" >> $uninstall
    echo "  $libdir/x86_64f/uv \\" >> $uninstall
    echo "  $libdir/x86_64f/uvbytes \\" >> $uninstall
fi
docfiles="RELNOTES f90_unix_dirent ieee_arithmetic TR f90_unix_env \
ieee_exceptions f90_gc f90_unix_errno ieee_features f90_iostat \
f90_unix_file iso_c_binding f90_kind f90_unix_io iso_fortran_env \
f90_preconn_io f90_unix_proc k-install f90_stat nagfor k-using f90_unix \
nagfmcheck nag_modules f90_unix_dir fpp upsf95 dbx90 nag_f2003 nag_f2008 \
np53_manual"
for i in $docfiles; do
    if [ -f $libdir/html/$i.html ]; then
        echo "  $libdir/html/$i.html \\" >> $uninstall
    fi
    if [ -f $libdir/doc/$i.pdf ]; then
        echo "  $libdir/doc/$i.pdf \\" >> $uninstall
    fi
done
echo "  $libdir/doc/RELNOTES \\" >> $uninstall
echo "  $libdir/doc/TR.txt \\" >> $uninstall
if [ -d n32 ]; then
    echo "  $libdir/n32/bytes $libdir/o32/bytes \\" >> $uninstall
fi
echo "  $libdir/forcomp \\" >> $uninstall
if [ -f $libdir/oforcomp ]; then
    echo "  $libdir/oforcomp \\" >> $uninstall
fi
echo "  $libdir/fpp \\" >> $uninstall
if [ -x chsrcnam ]; then
    echo "  $libdir/chsrcnam \\" >> $uninstall
fi
cd ..
echo $n . $c
$maybe_removetree lib
echo $n . $c
if [ $whoami = "root" ]; then
	for i in nagfor dbx90 upsf95 nagfmcheck; do
		if [ -x "$bindir/$i" ]; then
			$chown $owner $bindir/$i
			$chgrp $group $bindir/$i
		fi
	done
	$chown $owner $libdir
	$chgrp $group $libdir
fi
if [ $defaultman = "YES" ]; then
	for i in man/*.1
	do
	    $move $i $man1dir/`basename $i .1`.$man1suffix
	    echo "  $man1dir/`basename $i .1`.$man1suffix \\" >> $uninstall
	done
	for i in man/*.3
	do
	    $move $i $man3dir/`basename $i .3`.$man3suffix
	    echo "  $man3dir/`basename $i .3`.$man3suffix \\" >> $uninstall
	done
	if [ $whoami = "root" ]; then
		for i in man/*.1
		do
			$chown $owner $man1dir/`basename $i .1`.$man1suffix
			$chgrp $group $man1dir/`basename $i .1`.$man1suffix
		done
		for i in man/*.3
		do
			$chown $owner $man3dir/`basename $i .3`.$man3suffix
			$chgrp $group $man3dir/`basename $i .3`.$man3suffix
		done
	fi
fi
if [ -f man/nagfor.z ]; then
	for i in nagfor dbx90 fpp nagfmcheck; do
	    $move man/$i.z $cat1dir/$i.Z
	    echo "  $cat1dir/$i.Z \\" >> $uninstall
	    if [ $whoami = "root" ]; then
		$chown $owner $cat1dir/$i.Z
		$chgrp $group $cat1dir/$i.Z
	    fi
        done
	for i in f90_gc f90_unix f90_unix_file ieee_features \
                 f90_iostat f90_unix_dir f90_unix_io nag_modules \
		 f90_kind f90_unix_dirent f90_unix_proc \
                 f90_preconn_io f90_unix_env ieee_arithmetic \
	         f90_stat f90_unix_errno ieee_exceptions iso_fortran_env; do
	    $move man/$i.z $cat3dir/$i.Z
	    echo "  $cat3dir/$i.Z \\" >> $uninstall
	    if [ $whoami = "root" ]; then
		$chown $owner $cat3dir/$i.Z
		$chgrp $group $cat3dir/$i.Z
	    fi
	done
fi
echo "  $uninstall\"" >> $uninstall
echo "for i in \$files; do" >> $uninstall
echo "  rm -f \$i" >> $uninstall
echo "done" >> $uninstall
echo "cd /" >> $uninstall
echo "rmdir $libdir/html $libdir/doc" >> $uninstall
for i in $nagdirs; do
    echo "rmdir $i" >> $uninstall
done
chmod 755 $uninstall
if [ $whoami = "root" ]; then
    $chown $owner $uninstall
    $chgrp $group $uninstall
fi
$maybe_removetree man
find $libdir -type d -exec chmod 755 {} \;
echo
echo
echo NAG Fortran installation complete.
echo For the complete Compiler Manual, please see 
echo   $libdir/doc/np53_manual.pdf
echo
echo Thank you for installing NAG Fortran for the Sun SPARC.
echo
echo Here at NAG we\'d really like to know who\'s using this compiler
echo implementation, please take a few moments to email support@nag.co.uk
echo to record your interest in this platform, please quote the NAG
echo product code NPSOL53NA.
