/* ddbasic.h
 *
 * Basic header file for "double-double" precision library.
 *
 * Copyright 2008-2009 Nihon Numerical Algorithms Group KK, Tokyo, Japan.
 *
 * Release 5.2: $Id: ddbasic.h 1 2009-12-02 02:54:28Z Malcolm Cohen $
 */

#ifndef DDBASIC_H_INCLUDED
#define DDBASIC_H_INCLUDED

/*
 * Basic data types.
 */
typedef struct { double hi,lo; } DDReal;
typedef struct { DDReal re,im; } DDComplex;

enum {
  atodd_NO_ERROR=0, atodd_NO_MEMORY, atodd_ONLY_SIGN_FOUND,
  atodd_BAD_INPUT_REAL, atodd_TOO_MANY_DECIMAL_SYMBOLS,
  atodd_BAD_INPUT_EXPONENT, atodd_REAL_INPUT_OVERFLOW,
  atodd_REAL_INPUT_UNDERFLOW, atodd_OUTPUT_STRING_TOO_SHORT, atodd_IO_ERROR
};

DDReal CDECL __NAGf90_dd_DIVIDE_d(DDReal a,double b);
DDReal CDECL __NAGf90_dd_DIVIDE_dd(DDReal a,DDReal b);
DDComplex CDECL __NAGf90_dd_DIVIDE_ddc(DDReal,DDComplex);
DDReal CDECL __NAGf90_dd_TIMES_d(DDReal a,double b);
DDReal CDECL __NAGf90_dd_TIMES_dd(DDReal a,DDReal b);
DDReal CDECL __NAGf90_dd_pow_dd(DDReal x,DDReal y);
DDReal CDECL __NAGf90_dd_pow_i4(DDReal x,INT64 y);
DDComplex CDECL __NAGf90_ddc_DIVIDE_c(DDComplex x,Complex y);
DDComplex CDECL __NAGf90_ddc_DIVIDE_dc(DDComplex x,DComplex y);
DDComplex CDECL __NAGf90_ddc_DIVIDE_ddc(DDComplex x,DDComplex y);
DDComplex CDECL __NAGf90_ddc_pow_i4(DDComplex x,INT64 y);
DDComplex CDECL __NAGf90_ddc_pow_ddc(DDComplex x,DDComplex y);

/*
 * Elementary functions.
 */
DDReal CDECL __NAGf90_abs_ddc(const DDComplex *a);
DDReal CDECL __NAGf90_acos_dd(DDReal x);
DDReal CDECL __NAGf90_aint_dd(DDReal a);
DDReal CDECL __NAGf90_anint_dd(DDReal a);
DDReal CDECL __NAGf90_asin_dd(DDReal x);
DDReal CDECL __NAGf90_atan_dd(DDReal x);
DDReal CDECL __NAGf90_atan2_dd(DDReal y,DDReal x);
DDReal CDECL __NAGf90_ceiling_dd(DDReal x);
DDReal CDECL __NAGf90_copysign_dd(DDReal a,DDReal b);
DDReal CDECL __NAGf90_cos_dd(DDReal x);
DDComplex CDECL __NAGf90_cos_ddc(const DDComplex *x);
DDReal CDECL __NAGf90_cosh_dd(DDReal x);
DDReal CDECL __NAGf90_ddprod_d_d(double a,double b);
DDReal CDECL __NAGf90_exp_dd(DDReal x);
DDComplex CDECL __NAGf90_exp_ddc(const DDComplex *x);
int CDECL __NAGf90_exponent_dd(const DDReal *dd);
DDReal CDECL __NAGf90_floor_dd(DDReal x);
DDReal CDECL __NAGf90_fmod_dd(DDReal x,DDReal y);
DDReal CDECL __NAGf90_fraction_dd(DDReal x);
DDReal CDECL __NAGf90_log_dd(DDReal x);
DDComplex CDECL __NAGf90_log_ddc(const DDComplex *x);
DDReal CDECL __NAGf90_log10_dd(DDReal x);
DDReal CDECL __NAGf90_modulo_dd(DDReal,DDReal);
DDReal CDECL __NAGf90_nearest_dd(DDReal x,DDReal y);
DDReal CDECL __NAGf90_rrspacing_dd(const DDReal *x);
DDReal CDECL __NAGf90_scale_dd(DDReal x,int n);
DDReal CDECL __NAGf90_set_exponent_dd(DDReal q,Integer4 e);
DDReal CDECL __NAGf90_sin_dd(DDReal x);
DDComplex CDECL __NAGf90_sin_ddc(const DDComplex *x);
void CDECL __NAGf90_sincos_dd(DDReal a,DDReal *sine,DDReal *cosine);
DDReal CDECL __NAGf90_sinh_dd(DDReal x);
DDReal CDECL __NAGf90_spacing_dd(const DDReal *x);
DDReal CDECL __NAGf90_sqrt_dd(DDReal x);
DDComplex CDECL __NAGf90_sqrt_ddc(const DDComplex *x);
DDReal CDECL __NAGf90_tan_dd(DDReal x);
DDReal CDECL __NAGf90_tanh_dd(DDReal x);

#endif
