/* dope.h
 *
 * NAG Fortran 90/95/2003 Dope Vector definitions.
 *
 * Copyright 1992-2011 The Numerical Algorithms Group Ltd., Oxford, U.K.
 *
 * Malcolm Cohen, November 1992.
 *
 * Release 3: 1.1, 96/10/30
 *
 * Release 4: 1.1, 98/08/25
 *
 * Release 4.1: 1.2, 01/11/20
 *
 * Release 4.4: 1.3, 04/02/20
 *
 * Release 5.1: 1.2, 05/06/15
 *
 * Release 5.2: dope.h 1 2009-12-02 02:54:28Z Malcolm Cohen
 *
 * Release 5.3: $Id: dope.h 1209 2011-05-19 06:38:25Z Malcolm Cohen $
 */

#ifndef DOPE_H_INCLUDED
#define DOPE_H_INCLUDED

#ifdef OBJECT_SIZE_64
typedef INT64 dope_Offset;
#else
typedef Integer3 dope_Offset;
#endif
#if defined(CHRLEN_SIZE_64) && defined(OBJECT_SIZE_64)
typedef INT64 dope_Chrlen;
#else
typedef Integer3 dope_Chrlen;
#endif

typedef struct {
  Char *addr;
  dope_Chrlen len;
} Substring;

typedef struct {
  Char2 *addr;
  dope_Chrlen len;
} Substring2;

typedef struct {
  Char4 *addr;
  dope_Chrlen len;
} Substring4;

typedef struct {
  dope_Offset extent,mult,lower;
} Triplet;

/* Bounds Info record for contiguous variable-bounds arrays.
 */
typedef struct {
  int rank;
  dope_Offset offset;
  Triplet dim[7];
} BoundsInfo;

typedef struct {
  Pointer addr;
  dope_Offset offset;
  Triplet dim[1];
} Dope1;

typedef struct {
  Pointer addr;
  dope_Offset offset;
  Triplet dim[2];
} Dope2;

typedef struct {
  Pointer addr;
  dope_Offset offset;
  Triplet dim[3];
} Dope3;

typedef struct {
  Pointer addr;
  dope_Offset offset;
  Triplet dim[4];
} Dope4;

typedef struct {
  Pointer addr;
  dope_Offset offset;
  Triplet dim[5];
} Dope5;

typedef struct {
  Pointer addr;
  dope_Offset offset;
  Triplet dim[6];
} Dope6;

typedef struct {
  Pointer addr;
  dope_Offset offset;
  Triplet dim[7];
} Dope7;

typedef struct {
  Char *addr;
  dope_Chrlen len;
  dope_Offset offset;
  Triplet dim[1];
} ChDope1;

typedef struct {
  Char *addr;
  dope_Chrlen len;
  dope_Offset offset;
  Triplet dim[2];
} ChDope2;

typedef struct {
  Char *addr;
  dope_Chrlen len;
  dope_Offset offset;
  Triplet dim[3];
} ChDope3;

typedef struct {
  Char *addr;
  dope_Chrlen len;
  dope_Offset offset;
  Triplet dim[4];
} ChDope4;

typedef struct {
  Char *addr;
  dope_Chrlen len;
  dope_Offset offset;
  Triplet dim[5];
} ChDope5;

typedef struct {
  Char *addr;
  dope_Chrlen len;
  dope_Offset offset;
  Triplet dim[6];
} ChDope6;

typedef struct {
  Char *addr;
  dope_Chrlen len;
  dope_Offset offset;
  Triplet dim[7];
} ChDope7;

/* Dope vectors for old (non-PDT-capable) polymorphic array pointers. */

typedef struct {
  Dope1 dv;
  struct __NAGf90_signature *sig;
} PDope1;

typedef struct {
  Dope2 dv;
  struct __NAGf90_signature *sig;
} PDope2;

typedef struct {
  Dope3 dv;
  struct __NAGf90_signature *sig;
} PDope3;

typedef struct {
  Dope4 dv;
  struct __NAGf90_signature *sig;
} PDope4;

typedef struct {
  Dope5 dv;
  struct __NAGf90_signature *sig;
} PDope5;

typedef struct {
  Dope6 dv;
  struct __NAGf90_signature *sig;
} PDope6;

typedef struct {
  Dope7 dv;
  struct __NAGf90_signature *sig;
} PDope7;

/* Dope vectors for new (PDT-capable) polymorphic array pointers. */

typedef struct {
  Dope1 dv;
  struct __NAGf90_signature *sig;
  void *dtp;
} NPDope1;

typedef struct {
  Dope2 dv;
  struct __NAGf90_signature *sig;
  void *dtp;
} NPDope2;

typedef struct {
  Dope3 dv;
  struct __NAGf90_signature *sig;
  void *dtp;
} NPDope3;

typedef struct {
  Dope4 dv;
  struct __NAGf90_signature *sig;
  void *dtp;
} NPDope4;

typedef struct {
  Dope5 dv;
  struct __NAGf90_signature *sig;
  void *dtp;
} NPDope5;

typedef struct {
  Dope6 dv;
  struct __NAGf90_signature *sig;
  void *dtp;
} NPDope6;

typedef struct {
  Dope7 dv;
  struct __NAGf90_signature *sig;
  void *dtp;
} NPDope7;

/* Scalar CLASS(*) pointer. */

typedef struct {
  union { dope_Offset len; void *dtp; } x;
  dope_Offset size;
} __NAGf90_tpinfo;

struct __NAGf90_class_star {
  Pointer addr;
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
};

/* Dope vectors for CLASS(*) array pointers. */

typedef struct {
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
  Dope1 dv;
} __NAGf90_CSDope1;

typedef struct {
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
  Dope2 dv;
} __NAGf90_CSDope2;

typedef struct {
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
  Dope3 dv;
} __NAGf90_CSDope3;

typedef struct {
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
  Dope4 dv;
} __NAGf90_CSDope4;

typedef struct {
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
  Dope5 dv;
} __NAGf90_CSDope5;

typedef struct {
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
  Dope6 dv;
} __NAGf90_CSDope6;

typedef struct {
  struct __NAGf90_signature *sig;
  __NAGf90_tpinfo tp;
  Dope7 dv;
} __NAGf90_CSDope7;

/*
 * Miscellaneous i/o dope.
 */

typedef struct bsinfo_struct {
  int cdtype;
  dope_Offset pos,repeat;
  struct bsinfo_struct *dt;
} BS_Info;

typedef struct {
  int bs_version;
  dope_Offset bs_size,eltsize;
  BS_Info *unused;
} BS_Header;

#endif
