/* libfm.h
 *
 * Fortran Mathematical Functions.
 *
 * Copyright 2008-2012 The Numerical Algorithms Group Ltd., Oxford, U.K.
 *
 * Release 5.2: libfm.h 381 2010-08-12 08:47:54Z Malcolm Cohen
 *
 * Release 5.3: $Id: libfm.h 2068 2012-03-21 07:18:11Z Malcolm Cohen $
 */

#ifndef LIBFM_H_INCLUDED
#define LIBFM_H_INCLUDED

/*
 * NAG versions of maths functions.
 */
VPure double CDECL __NAGf90_acosd(double);
VPure float CDECL __NAGf90_acosf(float);
VPure double CDECL __NAGf90_asind(double);
VPure float CDECL __NAGf90_asinf(float);
VPure double CDECL __NAGf90_atand(double);
VPure float CDECL __NAGf90_atanf(float);
VPure double CDECL __NAGf90_atan2d(double,double);
VPure float CDECL __NAGf90_atan2f(float,float);
VPure double CDECL __NAGf90_cosd(double);
VPure double CDECL __NAGf90_cosd_s(double);
VPure float CDECL __NAGf90_cosf(float);
VPure double CDECL __NAGf90_coshd(double);
VPure float CDECL __NAGf90_coshf(float);
VPure float CDECL __NAGf90_coshf2(float);
VPure Complex CDECL __NAGf90_cpowc(Complex,Complex);
VPure Complex CDECL __NAGf90_cpowi4(Complex x,Integer4 y);
VPure DComplex CDECL __NAGf90_dcpowi4(DComplex x,Integer4 y);
VPure double CDECL __NAGf90_dpowi4(double x,Integer4 y);
VPure double CDECL __NAGf90_expd(double);
VPure float CDECL __NAGf90_expf(float);
VPure float CDECL __NAGf90_fpowi4(float x,Integer4 y);
Integer4 CDECL __NAGf90_i4powi4_g(const char *,int,Integer4,Integer4);
VPure double CDECL __NAGf90_logd(double);
VPure float CDECL __NAGf90_logf(float);
VPure double CDECL __NAGf90_log10d(double);
VPure float CDECL __NAGf90_log10f(float);
VPure float CDECL __NAGf90_powf(float x,float y);
VPure float CDECL __NAGf90_powf_c(float x,float y);
VPure double CDECL __NAGf90_powd(double x,double y);
VPure double CDECL __NAGf90_powd_c(double x,double y);
#ifdef QUADR
VPure QReal CDECL __NAGf90_qpowi4(QReal,Integer4);
#endif
VPure double CDECL __NAGf90_sind(double);
VPure double CDECL __NAGf90_sind_s(double);
VPure float CDECL __NAGf90_sinf(float);
VPure double CDECL __NAGf90_sinhd(double);
VPure float CDECL __NAGf90_sinhf(float);
void CDECL __NAGf90_sincos_d(double x,double *s,double *c);
void CDECL __NAGf90_sincos_r(float x,float *s,float *c);
VPure double CDECL __NAGf90_sqrtd(double);
VPure float CDECL __NAGf90_sqrtf(float);
VPure double CDECL __NAGf90_tand(double);
VPure float CDECL __NAGf90_tanf(float);
VPure double CDECL __NAGf90_tanhd(double x);
VPure float CDECL __NAGf90_tanhf(float);
VPure float CDECL __NAGf90_tanhf2(float);

/*
 * FDLIBM versions of maths functions.
 */
VPure double CDECL __NAGf90_acos(double);
VPure double CDECL __NAGf90_asin(double);
VPure double CDECL __NAGf90_atan(double);
VPure float CDECL __NAGf90_atan_f(float);
VPure double CDECL __NAGf90_atan2(double,double);
VPure float CDECL __NAGf90_atan2_f(float,float);
VPure double CDECL __NAGf90_cos(double x);
VPure float CDECL __NAGf90_cos_f(float);
VPure double CDECL __NAGf90_cosh(double);
VPure float CDECL __NAGf90_cosh_f(float);
VPure double CDECL __NAGf90_exp(double);
VPure double CDECL __NAGf90_fmod(double,double);
VPure float CDECL __NAGf90_fmod_f(float,float);
VPure double CDECL __NAGf90_log(double);
VPure double CDECL __NAGf90_log10(double);
VPure double CDECL __NAGf90_pow(double,double);
VPure float CDECL __NAGf90_pow_f(float,float);
VPure double CDECL __NAGf90_sin(double x);
VPure float CDECL __NAGf90_sin_f(float);
VPure double CDECL __NAGf90_sinh(double x);
VPure float CDECL __NAGf90_sinh_f(float);
VPure double CDECL __NAGf90_sqrt(double);
VPure double CDECL __NAGf90_tan(double);
VPure float CDECL __NAGf90_tan_f(float);
VPure double CDECL __NAGf90_tanh(double x);

/*
 * Other, not-quite-as-mathematical, functions.
 */
void CDECL __NAGf90_aspread(Pointer,Pointer,Offset,Offset,Offset);
Pure Chrlen CDECL __NAGf90_chindex1(const Char *string,const Char *substring,
                              Chrlen stringlen,Chrlen substringlen);
Pure Chrlen CDECL __NAGf90_chindex1b(const Char *string,const Char *substring,
                               int back,Chrlen stringlen,Chrlen substringlen);
Pure Chrlen CDECL __NAGf90_chindex2(const Char2 *string,const Char2 *substring,
                              Chrlen stringlen,Chrlen substringlen);
Pure Chrlen CDECL __NAGf90_chindex2b(const Char2 *string,const Char2 *substring,
                               int back,Chrlen stringlen,Chrlen substringlen);
Pure Chrlen CDECL __NAGf90_chindex4(const Char4 *string,const Char4 *substring,
                              Chrlen stringlen,Chrlen substringlen);
Pure Chrlen CDECL __NAGf90_chindex4b(const Char4 *string,const Char4 *substring,
                               int back,Chrlen stringlen,Chrlen substringlen);
Pure int CDECL __NAGf90_cselect(const Char *name,Chrlen name_len);
void CDECL __NAGf90_cshift_x(const char *,int,int,Pointer,const Triplet[],
                             Integer4,Integer4,Pointer,Offset);
void CDECL __NAGf90_cshift_xa(const char *,int,int,Pointer,const Triplet[],int,
                              Dope7 *,Integer4,Pointer,Offset,int);
void CDECL __NAGf90_eoshift_aa_line(const char *,int,int,Dope7*,Integer4,
                                    Integer4,int,Dope7*,Pointer,Offset);
void CDECL __NAGf90_eoshift_aaa_line(const char *,int,int,Dope7*,int,Dope7*,
                                     Integer4,int,Dope7*,Pointer,Offset,int);
void CDECL __NAGf90_eoshift_aas_line(const char *,int,int,Dope7*,int,Dope7*,
                                     Integer4,Pointer,Pointer,Offset,int);
void CDECL __NAGf90_eoshift_as_line(const char *,int,int,Dope7*,Integer4,
                                    Integer4,Pointer,Pointer,Offset);
void CDECL __NAGf90_eoshift_xaa_line(const char *,int,int,ChDope7*,Integer4,
                                     Integer4,int,ChDope7*,Pointer,Chrlen,int);
void CDECL __NAGf90_eoshift_xas_line(const char *,int,int,ChDope7*,Integer4,
                                     Integer4,Pointer,Chrlen,Pointer,Chrlen,
                                     int);
void CDECL __NAGf90_eoshift_xaaa_line(const char *,int,int,ChDope7*,int,Dope7*,
                                      Integer4,int,ChDope7*,Pointer,Chrlen,
                                      int,int);
void CDECL __NAGf90_eoshift_xaas_line(const char *,int,int,ChDope7*,int,Dope7*,
                                      Integer4,Pointer,Chrlen,Pointer,Chrlen,
                                      int,int);
Pure int CDECL __NAGf90_exponent_d(const double *x);
Pure int CDECL __NAGf90_exponent_r(const float *x);
Pure int CDECL __NAGf90_finddup_in_range8(Pointer v,Offset nelts,Offset step,
                                    Offset lb,Offset ub,INT64 *duplicate);
Pure int CDECL __NAGf90_finddup_in_range16(Pointer v,Offset nelts,Offset step,
                                     Offset lb,Offset ub,INT64 *duplicate);
Pure int CDECL __NAGf90_finddup_in_range32(Pointer v,Offset nelts,Offset step,
                                     Offset lb,Offset ub,INT64 *duplicate);
Pure int CDECL __NAGf90_finddup_in_range64(Pointer v,Offset nelts,Offset step,
                                     Offset lb,Offset ub,INT64 *duplicate);
VPure double CDECL __NAGf90_fraction_d(double x);
VPure float CDECL __NAGf90_fraction_r(float x);
void CDECL __NAGf90_mmr1(float *a,float *b,float *c,Offset n1,Offset n2,Offset n3);
void CDECL __NAGf90_mmr2(double *a,double *b,double *c,Offset n1,Offset n2,Offset n3);
void CDECL __NAGf90_mmdd(DDReal *a,DDReal *b,DDReal *c,Offset n1,Offset n2,Offset n3);
void CDECL __NAGf90_mmc1(Complex *a,Complex *b,Complex *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mmc2(DComplex *a,DComplex *b,DComplex *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mmddc(DDComplex *a,DDComplex *b,DDComplex *c,Offset n1,Offset n2,
                    Offset n3);
void CDECL __NAGf90_mmi1(Integer1 *a,Integer1 *b,Integer1 *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mmi2(Integer2 *a,Integer2 *b,Integer2 *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mmi3(Integer3 *a,Integer3 *b,Integer3 *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mmi4(Integer4 *a,Integer4 *b,Integer4 *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mml1(Logical1 *a,Logical1 *b,Logical1 *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mml2(Logical2 *a,Logical2 *b,Logical2 *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mml3(Logical *a,Logical *b,Logical *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mml4(Logical4 *a,Logical4 *b,Logical4 *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mvr1(float *a,float *b,float *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvr2(double *a,double *b,double *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvdd(DDReal *a,DDReal *b,DDReal *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvc1(Complex *a,Complex *b,Complex *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvc2(DComplex *a,DComplex *b,DComplex *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvddc(DDComplex *a,DDComplex *b,DDComplex *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvi1(Integer1 *a,Integer1 *b,Integer1 *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvi2(Integer2 *a,Integer2 *b,Integer2 *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvi3(Integer3 *a,Integer3 *b,Integer3 *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvi4(Integer4 *a,Integer4 *b,Integer4 *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvl1(Logical1 *a,Logical1 *b,Logical1 *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvl2(Logical2 *a,Logical2 *b,Logical2 *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvl3(Logical *a,Logical *b,Logical *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvl4(Logical4 *a,Logical4 *b,Logical4 *v,Offset n1,Offset n2);
void CDECL __NAGf90_vmr1(float *a,float *v,float *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmr2(double *a,double *v,double *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmdd(DDReal *a,DDReal *v,DDReal *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmc1(Complex *a,Complex *v,Complex *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmc2(DComplex *a,DComplex *v,DComplex *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmddc(DDComplex *a,DDComplex *v,DDComplex *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmi1(Integer1 *a,Integer1 *v,Integer1 *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmi2(Integer2 *a,Integer2 *v,Integer2 *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmi3(Integer3 *a,Integer3 *v,Integer3 *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmi4(Integer4 *a,Integer4 *v,Integer4 *c,Offset n2,Offset n3);
void CDECL __NAGf90_vml1(Logical1 *a,Logical1 *v,Logical1 *c,Offset n2,Offset n3);
void CDECL __NAGf90_vml2(Logical2 *a,Logical2 *v,Logical2 *c,Offset n2,Offset n3);
void CDECL __NAGf90_vml3(Logical *a,Logical *v,Logical *c,Offset n2,Offset n3);
void CDECL __NAGf90_vml4(Logical4 *a,Logical4 *v,Logical4 *c,Offset n2,Offset n3);
#ifdef QUADR
void CDECL __NAGf90_mmq(QReal *a,QReal *b,QReal *c,Offset n1,Offset n2,Offset n3);
void CDECL __NAGf90_mmqc(QComplex *a,QComplex *b,QComplex *c,Offset n1,Offset n2,
                   Offset n3);
void CDECL __NAGf90_mvq(QReal *a,QReal *b,QReal *v,Offset n1,Offset n2);
void CDECL __NAGf90_mvqc(QComplex *a,QComplex *b,QComplex *v,Offset n1,Offset n2);
void CDECL __NAGf90_vmq(QReal *a,QReal *v,QReal *c,Offset n2,Offset n3);
void CDECL __NAGf90_vmqc(QComplex *a,QComplex *v,QComplex *c,Offset n2,Offset n3);
#endif
VPure float CDECL __NAGf90_nearest_f(float x,float s);
VPure double CDECL __NAGf90_nearest_d(double x,double s);
Pure float CDECL __NAGf90_rrspacing_r(const float *);
Pure double CDECL __NAGf90_rrspacing_d(const double *);
VPure double CDECL __NAGf90_scale_d(double x,int i);
VPure float CDECL __NAGf90_scale_r(float,int);
VPure float CDECL __NAGf90_set_exponent_r(float f,INT64 i);
VPure double CDECL __NAGf90_set_exponent_d(double d,INT64 i);
Pure Char4 CDECL __NAGf90_sjis2_to_ucs4(Char2,Char4 *);
Pure float CDECL __NAGf90_spacing_r(const float *);
Pure double CDECL __NAGf90_spacing_d(const double *);
Pure int CDECL __NAGf90_str_eq_keyw(const Char *,Chrlen,const char *);
int CDECL __NAGf90_multiply_i64_by_i64(INT64 *xp,INT64 y);
Pure Chrlen CDECL __NAGf90_scan(const Char *,const Char *,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_scan2(const Char2 *,const Char2 *,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_scan2b(const Char2*,const Char2*,int,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_scan3(const Char*,const Char*,Logical,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_scan4(const Char4 *,const Char4 *,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_scan4b(const Char4*,const Char4*,int,Chrlen,Chrlen);
void CDECL __NAGf90_transpose(Pointer xaddr,Pointer yaddr,Offset n,Offset m,
                              Offset eltsize,Offset xstep0,Offset xstep1,
                              Offset ystep0,Offset ystep1);
void CDECL __NAGf90_transpose_ch1(Pointer xaddr,Pointer yaddr,Offset n,
                                  Offset m,Chrlen xlen,Chrlen ylen,
                                  Offset xstep0,Offset xstep1,Offset ystep0,
                                  Offset ystep1);
void CDECL __NAGf90_transpose_ch2(Pointer xaddr,Pointer yaddr,Offset n,
                                  Offset m,Chrlen xlen,Chrlen ylen,
                                  Offset xstep0,Offset xstep1,Offset ystep0,
                                  Offset ystep1);
void CDECL __NAGf90_transpose_ch4(Pointer xaddr,Pointer yaddr,Offset n,
                                  Offset m,Chrlen xlen,Chrlen ylen,
                                  Offset xstep0,Offset xstep1,Offset ystep0,
                                  Offset ystep1);
Pure Char2 CDECL __NAGf90_ucs4_to_sjis2(Char4);
Pure Chrlen CDECL __NAGf90_verify(const Char *,const Char *,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_verify2(const Char2 *,const Char2 *,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_verify2b(const Char2 *,const Char2 *,int,Chrlen,
                                    Chrlen);
Pure Chrlen CDECL __NAGf90_verify3(const Char *,const Char *,Logical,Chrlen,
                                   Chrlen);
Pure Chrlen CDECL __NAGf90_verify4(const Char4 *,const Char4 *,Chrlen,Chrlen);
Pure Chrlen CDECL __NAGf90_verify4b(const Char4 *,const Char4 *,int,Chrlen,
                                    Chrlen);

/*
 * Complex arithmetic support.
 */
VPure Complex CDECL __NAGf90_cdivc(Complex x,Complex y);
VPure DComplex CDECL __NAGf90_dcdivdc(DComplex x,DComplex y);
VPure DComplex CDECL __NAGf90_dcpowdc(DComplex,DComplex);
#ifdef QUADR
QComplex CDECL __NAGf90_qcdivc(QComplex,Complex);
QComplex CDECL __NAGf90_qcdivdc(QComplex,DComplex);
QComplex CDECL __NAGf90_qcdivqc(QComplex,QComplex);
QComplex CDECL __NAGf90_qcpowi4(QComplex,Integer4);
QComplex CDECL __NAGf90_qcpowqc(QComplex,QComplex);
#endif

/*
 * Complex intrinsics.
 */

float CDECL __NAGf90_abs_c(const Complex *);
double CDECL __NAGf90_abs_dc(const DComplex *);
Pure Complex CDECL __NAGf90_cos_c(const Complex *);
Pure DComplex CDECL __NAGf90_cos_dc(const DComplex *);
Pure Complex CDECL __NAGf90_exp_c(const Complex *);
Pure DComplex CDECL __NAGf90_exp_dc(const DComplex *);
Pure Complex CDECL __NAGf90_log_c(const Complex *);
Pure DComplex CDECL __NAGf90_log_dc(const DComplex *);
Pure Complex CDECL __NAGf90_sin_c(const Complex *);
Pure DComplex CDECL __NAGf90_sin_dc(const DComplex *);
Pure Complex CDECL __NAGf90_sqrt_c(const Complex *);
Pure DComplex CDECL __NAGf90_sqrt_dc(const DComplex *);
#ifdef QUADR
QReal CDECL __NAGf90_abs_qc(const QComplex *);
QComplex CDECL __NAGf90_cos_qc(const QComplex *);
QComplex CDECL __NAGf90_exp_qc(const QComplex *);
Integer3 CDECL __NAGf90_exponent_q(const QReal *);
QComplex CDECL __NAGf90_log_qc(const QComplex *);
QReal CDECL __NAGf90_nearest_q(QReal,QReal);
Pure QReal CDECL __NAGf90_rrspacing_q(const QReal*);
QReal CDECL __NAGf90_set_exponent_q(QReal,Integer4);
QComplex CDECL __NAGf90_sin_qc(const QComplex *);
Pure QReal CDECL __NAGf90_spacing_q(const QReal *);
QComplex CDECL __NAGf90_sqrt_qc(const QComplex *);
#endif

#ifdef Z9_HEX
double __NAGf90_checked_dpowi4(double x,Integer4 y,int *ifailp);
float __NAGf90_checked_fpowi4(float x,Integer4 y,int *ifailp);
static inline float __NAGf90_round_d_to_r(double d)
{
  UINT64 i = *(UINT64 *)&d;
  UINT32 r = i>>32;
  if (i&0x80000000 && (r&0x7fffffff)!=0x7fffffff) r++;
  return *(float*)&r;
}
#endif

#if defined(IBMZOS)
extern Char __NAGf90_achar[256];
extern Integer3 __NAGf90_iachar[256];
int __NAGf90_ascii_charcmp(const Char *a,const Char *b,Chrlen alen,Chrlen blen);
#endif

#endif /* LIBFM_H_INCLUDED */
