/* rts:nagfortran.h
 *
 * Runtime header file for NAG Fortran compiled code.
 *
 * Copyright 1991-2012 The Numerical Algorithms Group Ltd., Oxford, U.K.
 *
 * Malcolm Cohen, January 1991
 *
 * Version 2.0, November 1992.
 *
 * 1.17, 96/09/13
 *
 * Release 3: 1.15, 97/07/29
 *
 * Release 4: 1.10, 99/11/12
 *
 * Release 4.1: 1.33, 02/05/14
 *
 * Release 4.4: 1.49, 03/11/28
 *
 * Release 5.1: 1.25, 07/01/11
 *
 * Release 5.2: nagfortran.h 1 2009-12-02 02:54:28Z Malcolm Cohen
 *
 * Release 5.3: $Id: nagfortran.h 2303 2012-07-31 10:15:11Z Malcolm Cohen $
 */

#if defined(MSWIN32) || defined(__BORLANDC__)
/* stdcalls requires prototypes on everything? */
#define K_AND_R(type,name,ansi) extern type name ansi
#else
/* For backwards compatibility, we don't do that anywhere else. */
#define K_AND_R(type,name,ansi) extern type name()
#endif

#ifdef __GNUC__
#define VPure __attribute__((const))
#define Pure __attribute__((pure))
#define AllocFn __attribute__((malloc))
#define NoReturn __attribute__((noreturn))
#else
#define VPure
#define Pure
#define AllocFn
#define NoReturn
#endif

#ifdef WIN32
#define CDECL __cdecl
#else
#define CDECL
#endif

typedef char* Pointer;

#include <string.h>
#include <math.h>
#include <limits.h>
#ifdef MSWIN32
#include <float.h>
#endif
#ifdef SUNMATH
#include <sunmath.h>
#include <ieeefp.h>
#endif
#ifdef C99_FENV
#include <fenv.h>
int __NAGf90_ieee_get_flag(void);
void __NAGf90_ieee_set_flag(int);
#elif defined(IBM) || defined(LINUX) || defined(WIN32)
int CDECL __NAGf90_fpgetsticky(void);
int CDECL __NAGf90_fpsetsticky(int);
#ifndef IBM
int CDECL __NAGf90_fpgetround(void);
void CDECL __NAGf90_fpsetround(int);
#endif
#endif
int CDECL __NAGf90_fpgetmask(void);
void CDECL __NAGf90_fpsetmask(int);
int CDECL __NAGf90_get_underflow_mode(void);
void CDECL __NAGf90_set_underflow_mode(int);
void CDECL __NAGf90_fpgetstatus_clear_flags(int [4]);
void CDECL __NAGf90_fpsetstatus_merge_flags(const int [4]);
#if defined(SUN4) && defined(BC51_OPTION)
#include "sparc_names.h"
#endif

#if defined(MSWIN32) || defined(__BORLANDC__)
#define INT64 __int64
#define UINT64 unsigned __int64
#elif defined(WIN32) && !defined(INT64)
#define INT64 long long		/* For compiling user programs. */
#endif

#if defined(UCHARTYPE)		/* Override for broken C compilers. */
typedef UCHARTYPE Char;
#elif CHAR_MIN < 0
typedef unsigned char Char;
#else
typedef char Char;
#endif
typedef char Logical1;		/* Single-byte logical */
typedef short Logical2;
typedef int Logical;
typedef INT64 Logical4;
typedef signed char Integer1;		/* Single-byte signed integer */
typedef short Integer2;		/* Two-byte signed integer */
typedef int Integer;
typedef Integer Integer3;	/* For dope.h, sigh. */
typedef unsigned int UINT32;    /* True on all the machines we know about. */
typedef INT64 Integer4;
typedef unsigned short Char2;
typedef UINT32 Char4;
typedef float Real;
typedef double Double;
typedef struct {
  float re,im;
} Complex;
typedef struct {
  double re,im;
} DComplex;
#include "ddbasic.h"		/* Always define DDReal et cetera. */
#ifdef QUADR
#ifdef NATIVE_QUADR
typedef long double QReal;
typedef struct { QReal re,im; } QComplex;
#else
#define QReal DDReal
#define QComplex DDComplex
#endif
#endif
#include "dope.h"
typedef dope_Offset Offset;
typedef dope_Chrlen Chrlen;
#include "libfm.h"              /* Fortran Maths functions. */

typedef union {
  Integer i;
  Integer4 i4;
  Real r;
  Double d;
  Complex c;
  DComplex dc;
#ifdef QUADR
  QReal q;
  QComplex qc;
#endif
  Logical l;
} Multiple;			/* F77 storage-associated ENTRY. */

#if defined(_HPUX_SOURCE)
double cabs();
#elif defined(SGI)
float fsqrt(float),fexp(float),flog(float),flog10(float);
float powf(float,float),facos(float),fasin(float),fatan(float);
float fatan2(float,float),fceil(float),fcos(float),fcosh(float);
float ffloor(float),fsin(float),fsinh(float),ftan(float),ftanh(float);
#endif

/* Out-of-line Complex division.
 */
VPure Complex CDECL __NAGf90_rdivc(Real x,Complex y);
VPure DComplex CDECL __NAGf90_dpdivdc(Double x,DComplex y);
VPure DComplex CDECL __NAGf90_cdivdc(Complex x,DComplex y);
VPure DComplex CDECL __NAGf90_dpdivc(Double x,Complex y);
VPure DComplex CDECL __NAGf90_dcdivc(DComplex x,Complex y);

/* Out-of-line Exponentiation -- integer exponent.
 */
VPure Complex CDECL __NAGf90_cpowi(Complex x,Integer y);
VPure Double CDECL __NAGf90_dpowi(Double x,Integer y);
VPure DComplex CDECL __NAGf90_dcpowi(DComplex x,Integer y);
VPure Real CDECL __NAGf90_fpowi(Real x,Integer y);
VPure Integer CDECL __NAGf90_ipowi(Integer x,Integer y);
VPure Integer CDECL __NAGf90_ipowi_g(const char *,int,Integer,Integer);
VPure Integer4 CDECL __NAGf90_i4powi4(Integer4 x,Integer4 y);

/* Other out-of-line exponentiation */
VPure Real CDECL __NAGf90_ipowr(Integer x,Real y);
VPure Double CDECL __NAGf90_ipowd(Integer x,Double y);
VPure Real CDECL __NAGf90_i4powr(Integer4 x,Real y);
VPure Double CDECL __NAGf90_i4powd(Integer4 x,Double y);

/* Array Temporaries */
typedef struct {
  Integer1 *addr;
  Offset extent[7];
} ArrayTemp_Integer1;
typedef struct {
  Integer2 *addr;
  Offset extent[7];
} ArrayTemp_Integer2;
typedef struct {
  Integer *addr;
  Offset extent[7];
} ArrayTemp_Integer;
typedef struct {
  Integer4 *addr;
  Offset extent[7];
} ArrayTemp_Integer4;
typedef struct {
  Real *addr;
  Offset extent[7];
} ArrayTemp_Real;
typedef struct {
  Double *addr;
  Offset extent[7];
} ArrayTemp_Double;
typedef struct {
  Complex *addr;
  Offset extent[7];
} ArrayTemp_Complex;
typedef struct {
  DComplex *addr;
  Offset extent[7];
} ArrayTemp_DComplex;
#if defined(QUADR) && defined(NATIVE_QUADR)
typedef struct {
  QReal *addr;
  Offset extent[7];
} ArrayTemp_QReal;
typedef struct {
  QComplex *addr;
  Offset extent[7];
} ArrayTemp_QComplex;
#endif
typedef struct {
  DDReal *addr;
  Offset extent[7];
} ArrayTemp_DDReal;
typedef struct {
  DDComplex *addr;
  Offset extent[7];
} ArrayTemp_DDComplex;
typedef struct {
  Logical1 *addr;
  Offset extent[7];
} ArrayTemp_Logical1;
typedef struct {
  Logical2 *addr;
  Offset extent[7];
} ArrayTemp_Logical2;
typedef struct {
  Logical *addr;
  Offset extent[7];
} ArrayTemp_Logical;
typedef struct {
  Logical4 *addr;
  Offset extent[7];
} ArrayTemp_Logical4;
typedef struct {
  Char *addr;
  Chrlen len;
  Offset extent[7];
} ArrayTemp_Char;
typedef ArrayTemp_Char ArrayTemp_Derived; /* Same for any derived type. */
typedef struct {
  Char2 *addr;
  Chrlen len;
  Offset extent[7];
} ArrayTemp_Char2;
typedef struct {
  Char4 *addr;
  Chrlen len;
  Offset extent[7];
} ArrayTemp_Char4;

typedef struct __NAGf90_atmp_list_elt_type {
  ArrayTemp_Integer atmp;
  struct __NAGf90_atmp_list_elt_type *next;
} ATmp_List_Elt;

typedef struct __NAGf90_atmp_list_ch_elt_type {
  ArrayTemp_Char atmp;
  struct __NAGf90_atmp_list_ch_elt_type *next;
} ATmp_List_Ch_Elt;

typedef struct {
  ATmp_List_Elt *list;
} ATmp_List_Type;

void CDECL __NAGf90_atmplist_append(ATmp_List_Type*,Pointer);
void CDECL __NAGf90_atmplist_append_ch(ATmp_List_Type*,ArrayTemp_Char*);
ATmp_List_Type CDECL __NAGf90_atmplist_create(void);
void CDECL __NAGf90_atmplist_finish(ATmp_List_Type);
void CDECL __NAGf90_atmplist_step(ATmp_List_Type*,Pointer);
void CDECL __NAGf90_atmplist_step_ch(ATmp_List_Type*,ArrayTemp_Char*);
void CDECL __NAGf90_atmplist_elt(ATmp_List_Type,Pointer,int);

/* Non-contig polymorphic array to contig polymorphic array copy. */
void CDECL __NAGf90_polyacopy(Pointer src,int rank,Pointer dst,Offset eltsize);

/* Character Operations */

void CDECL __NAGf90_chasgn(Char *,Chrlen,const Char *,Chrlen);
Substring CDECL __NAGf90_chcat(const Char *,Chrlen,const Char *,Chrlen);
ArrayTemp_Char CDECL __NAGf90_chcat_as(int,const ChDope7 *,const Char *,Chrlen);
ArrayTemp_Char CDECL __NAGf90_chcat_sa(int,const Char *,Chrlen,const ChDope7*);
ArrayTemp_Char CDECL __NAGf90_chcat_aa(int,const ChDope7*,const ChDope7*);
int CDECL __NAGf90_charcmp(const Char *,const Char *,Chrlen,Chrlen);
void CDECL __NAGf90_chfree(Substring *ssp);
void CDECL __NAGf90_line_chfree(Substring *ssp,int lineno,const char *fn);
Substring CDECL __NAGf90_cat_a2s(int,const ChDope7*);

void CDECL __NAGf90_vec_ch_asgn_array_ch_g(const char *,int,ArrayTemp_Char,int,
                                           const ChDope7 *);
void CDECL __NAGf90_array_ch_asgn_array_ch(int,const ChDope7*,int,
                                           const ChDope7*);
void CDECL __NAGf90_array_ch_asgn_array_ch_g(const char *,int,int,
                                             const ChDope7*,int,const ChDope7*);

/* NAMELIST */

typedef struct {
  int ncomponents;
  struct type_component_description *component;
} Typedesc;

typedef struct type_component_description {
  char *name;
  Offset offset;
  Offset *dims;
  int cdtype;
  Chrlen chrlen;
  Typedesc *dtdesc;
} Compdesc;

typedef struct {
  char *name;
  Pointer addr;
  Offset *dims;
  int cdtype;
  Chrlen chrlen;
  Typedesc *dtdesc;
} Vardesc;

typedef struct {
  char *name;
  Vardesc **var;
  int nvars;
} Namelist;

/* Error Handling */

NoReturn void CDECL __NAGf90_badptr0_line(const char *,int,Pointer);
NoReturn void CDECL __NAGf90_badptr1_line(const char *,int,const char *,
                                          Pointer);
NoReturn void CDECL __NAGf90_badptr2a_line(const char *,int,const char *);
NoReturn void CDECL __NAGf90_badptr3_line(const char *,int,Pointer,char *,int);

extern const char *__NAGf90_filename;
extern int __NAGf90_lineno;
void CDECL __NAGf90_enter_pu(const char *routine);
void CDECL __NAGf90_exit_pu(void);

NoReturn void CDECL __NAGf90_rterr(const char *,int,int,...);

/* Confused-Level Storage Allocation */

void CDECL __NAGf90_DeAlloc_o(void *,Offset);
void CDECL __NAGf90_line_DeAlloc_o(void *,Offset,int,const char *);
Pointer CDECL __NAGf90_ReAlloc(Pointer,Offset,Offset,Offset);
Pointer CDECL __NAGf90_line_ReAlloc(Pointer pointer,Offset eltsize,
                                    Offset oldsize,Offset newsize,int lineno,
                                    const char *fn);

/* Top Level Allocation Routines */

BoundsInfo CDECL __NAGf90_make_info(int,...);
void CDECL __NAGf90_Deallocate_ca(int,ChDope7*,Integer*);
Pointer AllocFn CDECL __NAGf90_Allocate_aa(Offset,Integer *);
void CDECL __NAGf90_Deallocate_aa(Pointer,const Triplet[],int,Offset,Integer*);

Pointer AllocFn CDECL __NAGf90_Allocate_s(Integer,Integer*);
Pointer AllocFn CDECL __NAGf90_Allocate_zs(Integer,Integer*);
void CDECL __NAGf90_Deallocate_p(Pointer,Offset,Integer*);
Pointer AllocFn CDECL __NAGf90_Allocate_i(BoundsInfo,Integer,Integer*);
Pointer AllocFn CDECL __NAGf90_Allocate_zi(BoundsInfo,Integer,Integer*);
void CDECL __NAGf90_Deallocate_i(Pointer,BoundsInfo,Integer,Integer*);
Pointer AllocFn CDECL __NAGf90_Alloc_i(BoundsInfo,Integer);
Pointer AllocFn CDECL __NAGf90_Alloc_zi(BoundsInfo,Integer);
Pointer AllocFn CDECL __NAGf90_Alloc_var(Offset);
Pointer AllocFn CDECL __NAGf90_Alloc_zvar(Offset);
Pointer AllocFn CDECL __NAGf90_Alloc_atmp(Offset);
Pointer AllocFn CDECL __NAGf90_Alloc_zatmp(Offset);

Pointer AllocFn CDECL __NAGf90_line_Alloc_atmp(Offset,int,const char *);
Pointer AllocFn CDECL __NAGf90_line_Alloc_i(BoundsInfo,Integer,int,
                                            const char *);
Pointer AllocFn CDECL __NAGf90_line_Alloc_var(Offset,int,const char *);
Pointer AllocFn CDECL __NAGf90_line_Alloc_zatmp(Offset,int,const char *);
Pointer AllocFn CDECL __NAGf90_line_Alloc_zi(BoundsInfo,Integer,int,
                                             const char *);
Pointer AllocFn CDECL __NAGf90_line_Alloc_zvar(Offset,int,const char *);
Pointer AllocFn CDECL __NAGf90_line_Allocate_aac(Offset size,Integer *stat,
                                                 int lineno,const char *fn);
Pointer AllocFn CDECL __NAGf90_line_Allocate_s(Offset,Integer*,int,
                                               const char *);
Pointer AllocFn CDECL __NAGf90_line_Allocate_zs(Offset,Integer*,int,
                                                const char *);
void CDECL __NAGf90_line_Deallocate_aa(Pointer addr,Triplet dim[],int rank,
                                       Integer eltsize,Integer *stat,
                                       int lineno,const char *fn);
void CDECL __NAGf90_line_Deallocate_i(Pointer,BoundsInfo,Integer,Integer *,int,
                                      const char *);
void CDECL __NAGf90_line_Deallocate_s(Pointer,Offset,Integer *,int,
                                      const char *);
Pointer CDECL __NAGf90_Allocate_as(Offset,Integer *);
Pointer CDECL __NAGf90_Allocate_p(Offset,Integer *);
Pointer CDECL __NAGf90_Allocate_zas(Offset,Integer *);
Pointer CDECL __NAGf90_Allocate_zp(Offset,Integer *);
Pointer CDECL __NAGf90_line_Allocate_as(Offset,Integer *,int,const char *);
Pointer CDECL __NAGf90_line_Allocate_p(Offset,Integer *,int,const char *);
Pointer CDECL __NAGf90_line_Allocate_zas(Offset,Integer *,int,const char *);
Pointer CDECL __NAGf90_line_Allocate_zp(Offset,Integer *,int,const char *);

/* 5.2 memory allocation */

void CDECL __NAGf90_aDeAlloc(Pointer,Offset);
void CDECL __NAGf90_laDeAlloc(const char *,int,Pointer,Offset);
Pointer CDECL __NAGf90_aAlloc(Offset);
Pointer CDECL __NAGf90_laAlloc(Offset,int,const char *);
Pointer CDECL __NAGf90_zaAlloc(Offset);
Pointer CDECL __NAGf90_lzaAlloc(Offset,int,const char *);

/* Intrinsic Functions */

K_AND_R(Real CDECL,	__NAGf90_aint_r,	(const Real *r));
K_AND_R(Double CDECL,	__NAGf90_aint_d,	(const Double *));

K_AND_R(Real CDECL, __NAGf90_anint_r, (const Real*));
K_AND_R(Double CDECL, __NAGf90_anint_d, (const Double *));

void CDECL __NAGf90_date_and_time_line(const char *,int,Char *,Char *,Char *,
				       Dope1 *,Chrlen,Chrlen,Chrlen);
void CDECL __NAGf90_date_and_time_i4_line(const char *,int,Char *,Char*,Char*,
					  Dope1 *,Chrlen,Chrlen,Chrlen);

Pure Complex CDECL __NAGf90_exp_imag(const Real *);
Pure DComplex CDECL __NAGf90_exp_dimag(const Double *);

VPure Integer CDECL __NAGf90_ishftc(Integer,Integer,Integer,Integer);
VPure Integer4 CDECL __NAGf90_ishftc_i4(Integer4,Integer4,Integer,Integer);

K_AND_R(Real CDECL, __NAGf90_mod_r,		(Real*, Real*));
K_AND_R(Double CDECL, __NAGf90_mod_d,		(Double *, Double*));

Pure Chrlen CDECL __NAGf90_len_trim1(const Char *,Chrlen);

Pure Integer CDECL __NAGf90_nint_r_i(Real *);
Pure Integer CDECL __NAGf90_nint_d_i(Double *);

void CDECL __NAGf90_pack_line(const char *,int,ArrayTemp_Integer *,int,
                              const Dope7 *,int,const Dope7 *,const Dope1 *,
                              Offset);
void CDECL __NAGf90_l1_pack_line(const char *,int,ArrayTemp_Integer *,int,
                                 const Dope7 *,int,const Dope7 *,const Dope1 *,
                                 Offset);
void CDECL __NAGf90_l2_pack_line(const char *,int,ArrayTemp_Integer *,int,
                                 const Dope7 *,int,const Dope7 *,const Dope1 *,
                                 Offset);
void CDECL __NAGf90_l4_pack_line(const char *,int,ArrayTemp_Integer *,int,
                                 const Dope7 *,int,const Dope7 *,
                                 const Dope1 *,Offset);
void CDECL __NAGf90_chpack_line(const char *,int,ArrayTemp_Char *,int,
                                const ChDope7 *,int,const Dope7 *,
                                const ChDope1*);
void CDECL __NAGf90_ch2pack_line(const char *,int,ArrayTemp_Char2 *,int,
                                 const ChDope7 *,int,const Dope7 *,
                                 const ChDope1*);
void CDECL __NAGf90_ch4pack_line(const char *,int,ArrayTemp_Char4 *,int,
                                 const ChDope7 *,int,const Dope7 *,
                                 const ChDope1*);
void CDECL __NAGf90_dtpack_line(const char *,int,ArrayTemp_Derived *,int,
                                const Dope7 *,int,const Dope7 *,const Dope1 *,
                                Offset);
void CDECL __NAGf90_l1_chpack_line(const char *,int,ArrayTemp_Char *,int,
                                   const ChDope7 *,int,const Dope7 *,
                                   const ChDope1 *);
void CDECL __NAGf90_l1_ch2pack_line(const char *,int,ArrayTemp_Char2 *,int,
                                    const ChDope7 *,int,const Dope7 *,
                                    const ChDope1 *);
void CDECL __NAGf90_l1_ch4pack_line(const char *,int,ArrayTemp_Char4 *,int,
                                    const ChDope7 *,int,const Dope7 *,
                                    const ChDope1 *);
void CDECL __NAGf90_l1_dtpack_line(const char *,int,ArrayTemp_Derived *,int,
                                   const Dope7 *,int,const Dope7 *,
                                   const Dope1 *,Offset);
void CDECL __NAGf90_l2_chpack_line(const char *,int,ArrayTemp_Char *,int,
                                   const ChDope7 *,int,const Dope7*,
                                   const ChDope1 *);
void CDECL __NAGf90_l2_ch2pack_line(const char *,int,ArrayTemp_Char2 *,int,
                                    const ChDope7 *,int,const Dope7*,
                                    const ChDope1 *);
void CDECL __NAGf90_l2_ch4pack_line(const char *,int,ArrayTemp_Char4 *,int,
                                    const ChDope7 *,int,const Dope7*,
                                    const ChDope1 *);
void CDECL __NAGf90_l2_dtpack_line(const char *,int,ArrayTemp_Derived *,int,
                                   const Dope7 *,int,const Dope7 *,
                                   const Dope1 *,Offset);
void CDECL __NAGf90_l4_chpack_line(const char *,int,ArrayTemp_Char *,int,
                                   const ChDope7 *,int,const Dope7 *,
                                   const ChDope1 *);
void CDECL __NAGf90_l4_ch2pack_line(const char *,int,ArrayTemp_Char2 *,int,
                                    const ChDope7 *,int,const Dope7 *,
                                    const ChDope1 *);
void CDECL __NAGf90_l4_ch4pack_line(const char *,int,ArrayTemp_Char4 *,int,
                                    const ChDope7 *,int,const Dope7 *,
                                    const ChDope1 *);
void CDECL __NAGf90_l4_dtpack_line(const char *,int,ArrayTemp_Derived *,int,
                                   const Dope7 *,int,const Dope7 *,
                                   const Dope1 *,Offset);

void CDECL __NAGf90_init_random52(void);
Real CDECL __NAGf90_random52_r(void);
Double CDECL __NAGf90_random52_d(void);
DDReal CDECL __NAGf90_random52_dd(void);
void CDECL __NAGf90_random52a_r(int,Dope7*);
void CDECL __NAGf90_random52a_d(int,Dope7*);
void CDECL __NAGf90_random52a_dd(int,Dope7*);
void CDECL __NAGf90_random52v_r(Real*,Offset);
void CDECL __NAGf90_random52v_d(Double*,Offset);
void CDECL __NAGf90_random52v_dd(DDReal*,Offset);
void CDECL __NAGf90_random_seed_size52(Integer*);
void CDECL __NAGf90_put_random_seed52(Dope1*);
void CDECL __NAGf90_get_random_seed52(const Dope1*);
void CDECL __NAGf90_random_seed52(Integer*,Dope1*,Dope1*);

void CDECL __NAGf90_reshape_line(const char *,int,ArrayTemp_Integer*,int,
                                 const Dope7*,const Dope1*,int,const Dope7 *,
                                 const Dope1 *,Offset);
void CDECL __NAGf90_chreshape_line(const char *,int,ArrayTemp_Char *,int,
                                   const ChDope7 *,const Dope1 *,int,
                                   const ChDope7 *,const Dope1 *);
void CDECL __NAGf90_ch2reshape_line(const char *,int,ArrayTemp_Char2 *,int,
                                    const ChDope7 *,const Dope1 *,int,
                                    const ChDope7 *,const Dope1 *);
void CDECL __NAGf90_ch4reshape_line(const char *,int,ArrayTemp_Char4 *,int,
                                    const ChDope7 *,const Dope1 *,int,
                                    const ChDope7 *,const Dope1 *);
void CDECL __NAGf90_dtreshape_line(const char *,int,ArrayTemp_Derived*,int,
                                   const Dope7*,const Dope1*,int,const Dope7 *,
                                   const Dope1 *,Offset);

VPure Integer CDECL __NAGf90_iselect64(Integer4);
VPure Integer CDECL __NAGf90_iselectb64(Integer4);
VPure Integer CDECL __NAGf90_rselect64(Integer4,Integer4);
VPure Integer CDECL __NAGf90_rselectb64(Integer4,Integer4);
VPure Integer CDECL __NAGf90_ieee_rselect64(Integer4,Integer4);
VPure Integer CDECL __NAGf90_ieee_rselectb64(Integer4,Integer4);

Pure Real CDECL __NAGf90_sign_r(Real *,Real *);
VPure Real CDECL __NAGf90_sign90_r(Real,Real);

Pure Double CDECL __NAGf90_sign_d(Double *d,Double *);
VPure Double CDECL __NAGf90_sign90_d(Double,Double);

Pure Offset CDECL __NAGf90_size_ptr(int,const Dope7*);
Pure Offset CDECL __NAGf90_size_cptr(int,const ChDope7*);
VPure Offset CDECL __NAGf90_size_info(BoundsInfo);
Pure Offset CDECL __NAGf90_size_vector(/* ArrayTemp_Integer */);
VPure Offset CDECL __NAGf90_size_cvector(ArrayTemp_Char);
Pure Offset CDECL __NAGf90_extent_ptr(int,const Dope7*,Integer);
Pure Offset CDECL __NAGf90_extent_cptr(int,const ChDope7*,Integer);
VPure Offset CDECL __NAGf90_extent_info(BoundsInfo,Integer);
Pure Offset CDECL __NAGf90_extent_vector(/* ArrayTemp_Integer,int */);
VPure Offset CDECL __NAGf90_extent_cvector(ArrayTemp_Char,int);

Pure Integer CDECL __NAGf90_extent_fixed(int,int,...); /* NOT 64-BIT */
Pure Integer CDECL __NAGf90_lbound_fixed(int,int,...); /* NOT 64-BIT */

void CDECL __NAGf90_spread_line(const char *,int,ArrayTemp_Integer *,int,
                                Dope7 *,Integer4,Integer4,Offset);
void CDECL __NAGf90_spread_pa_line(const char *,int,NPDope7 *,int,
                                   const NPDope7 *,Integer4,Integer4,Offset);
void CDECL __NAGf90_spread_pau_line(const char *,int,__NAGf90_CSDope7 *,int,
                                    const __NAGf90_CSDope7 *,Integer4,Integer4,
                                    Offset);
void CDECL __NAGf90_spread_ps_line(const char *,int,NPDope1 *,int,
                                   const struct __NAGf90_class_star *,Integer4,
                                   Integer4,Offset);
void CDECL __NAGf90_spread_psu_line(const char *,int,__NAGf90_CSDope1 *,int,
                                    const struct __NAGf90_class_star *,Integer4,
                                    Integer4,Offset);
void CDECL __NAGf90_dtspread_line(const char *,int,ArrayTemp_Derived *,int,
                                  const Dope7 *,Integer4,Integer4,Offset);
void CDECL __NAGf90_chspread_line(const char *,int,ArrayTemp_Char*,int,
                                  const ChDope7*,Integer4,Integer4);
void CDECL __NAGf90_ch2spread_line(const char *,int,ArrayTemp_Char2*,int,
                                   const ChDope7*,Integer4,Integer4);
void CDECL __NAGf90_ch4spread_line(const char *,int,ArrayTemp_Char4*,int,
                                   const ChDope7*,Integer4,Integer4);

void CDECL __NAGf90_system_clock(Integer*,Integer*,Integer*);

Integer CDECL __NAGf90_ubound_vector(int,Integer3[7],Integer);
VPure Integer CDECL __NAGf90_ubound_ainfo(BoundsInfo,Integer);
Integer CDECL __NAGf90_ubound_fixed(int,int,...);
Integer CDECL __NAGf90_ubound_afixed(int,int,...);

/*
 * Main program initialisation/termination.
 *
 * Note these do not start with __NAG.
 */
#ifdef WIN32
void CDECL __NAGf90_init_dll(int,char*[]);
void CDECL __NAGf90_init1_dll(int,char*[]);
void CDECL __NAGf90_init2(int,char*[]);
#endif
void CDECL f90_init(int,char*[]);
void CDECL f90_init1(int,char*[]);
void CDECL f90_init2(int,char*[]);
NoReturn void CDECL f90_finish(int);

NoReturn void CDECL __NAGf90_stop_line(const char *,int,const char *);
void CDECL __NAGf90_pause(const char *);

extern int __NAGf90_popcnt_table[256];

#ifdef QUADR
QReal CDECL __NAGf90_aint_q(const QReal*);
QReal CDECL __NAGf90_anint_q(const QReal*);
QComplex CDECL __NAGf90_cdivqc(Complex,QComplex);
void CDECL __NAGf90_cpu_time_q(QReal *);
QComplex CDECL __NAGf90_dcdivqc(DComplex,QComplex);
QReal CDECL __NAGf90_fraction_q(QReal);
QReal CDECL __NAGf90_max_q(int n,...);
QReal CDECL __NAGf90_min_q(int n,...);
QComplex CDECL __NAGf90_qcpowi(QComplex,Integer);
QComplex CDECL __NAGf90_qdivc(QReal,Complex);
QComplex CDECL __NAGf90_qdivdc(QReal,DComplex);
QComplex CDECL __NAGf90_qdivqc(QReal,QComplex);
QReal CDECL __NAGf90_qpowi(QReal,Integer);
QReal CDECL __NAGf90_random52_q(void);
void CDECL __NAGf90_random52a_q(int,Dope7*);
void CDECL __NAGf90_random52v_q(QReal*,Offset);
QReal CDECL __NAGf90_sign_q(const QReal*,const QReal*);
QReal CDECL __NAGf90_sign90_q(QReal,QReal);
#endif

void CDECL __NAGf90_cpu_time(Real *);
void CDECL __NAGf90_cpu_time_d(Double *);

/* New memory allocation "stuff" */

#define __NAGf90_NQL 120        /* Number of Quick Lists */
typedef struct {
  Pointer tail;
  int remaining;
} __NAGf90_TailInfo;

extern Pointer __NAGf90_allocatable_ql2[__NAGf90_NQL];
extern __NAGf90_TailInfo __NAGf90_allocatable_tail2;
extern Pointer __NAGf90_ordinary_ql2[__NAGf90_NQL];
extern __NAGf90_TailInfo __NAGf90_ordinary_tail2;
extern Pointer __NAGf90_pointer_ql2[__NAGf90_NQL];
extern __NAGf90_TailInfo __NAGf90_pointer_tail2;
extern int __NAGf90_fast_deallocation_allowed;

void CDECL __NAGf90_associate_pointer(Pointer p_addr,size_t t_size);
void CDECL __NAGf90_destroy_pointer(Pointer p_addr);
void CDECL __NAGf90_disassociate_target_because(Pointer t_addr,size_t t_size,
                                                int was_deallocated,int lineno,
                                                const char *filename,
						const char *puname);
void CDECL __NAGf90_disassociate_target(Pointer t_addr,size_t t_size);
void CDECL __NAGf90_cdangle_Deallocate_i(Pointer,BoundsInfo,Offset,Integer *,
                                         int,int,const char *,const char *);
void CDECL __NAGf90_cdangle_Deallocate_aac(Pointer,Triplet[],int,Offset,
                                           Integer *,int,int,const char *,
                                           const char *);
void CDECL __NAGf90_cdangle_Deallocate_s(Pointer,Offset,Integer *,int,int,
                                         const char *);
void CDECL __NAGf90_cdangling_aDeAlloc(const char *,int,Pointer,Offset,
                                       const char *,int);
int CDECL __NAGf90_pagecolour(Pointer pointer,Offset size);

NoReturn void CDECL __NAGf90_pu_bad_cdtype_line(const char *,int,const char *,
                                                int,int);

void CDECL __NAGf90_stat_errmsg(int statval,Char *errmsg,Chrlen len);

/* New Fortran 2003 intrinsics. */

void CDECL __NAGf90_getarg(Integer *number,Char *value,Integer *length,
                           Integer *status,Chrlen value_len);
void CDECL __NAGf90_getcmd(Char *command,Integer *length,Integer *status,
			   Chrlen command_len);
void CDECL __NAGf90_getenv(const Char *name,Char *value,Integer *length,
                           Integer *status,Logical *trim_name,Chrlen name_len,
                           Chrlen value_len);
int CDECL __NAGf90_iargc(void);

/* I/O. */

typedef struct __NAGf90_io_context_struct IO_context;
typedef struct __NAGf90_internal_file_struct Internal_file;

void CDECL __NAGf90_array_ifilex(Internal_file *,int,const ChDope7 *,int);
int CDECL __NAGf90_backspacef(int,Char *,Chrlen,int);
int CDECL __NAGf90_backspacef_line(const char *,int,int,Char *,Chrlen,int);
int CDECL __NAGf90_closef(int,const Char *,Chrlen,Char *,Chrlen,int);
int CDECL __NAGf90_closef_line(const char *,int,int,const Char *,Chrlen,Char *,
                               Chrlen,int);
int CDECL __NAGf90_do_read_namelist(IO_context *,Namelist *namelist);
int CDECL __NAGf90_do_write_namelist(IO_context *,const Namelist *namelist);
int CDECL __NAGf90_dusty_fmt_read_c(IO_context *,Complex *var);
int CDECL __NAGf90_dusty_fmt_read_d(IO_context *,Double *var);
int CDECL __NAGf90_dusty_fmt_read_dc(IO_context *,DComplex *var);
int CDECL __NAGf90_dusty_fmt_read_dd(IO_context *,DDReal *var);
int CDECL __NAGf90_dusty_fmt_read_ddc(IO_context *,DDComplex *var);
int CDECL __NAGf90_dusty_fmt_read_i(IO_context *,Integer *var);
int CDECL __NAGf90_dusty_fmt_read_i1(IO_context *,Integer1 *var);
int CDECL __NAGf90_dusty_fmt_read_i2(IO_context *,Integer2 *var);
int CDECL __NAGf90_dusty_fmt_read_i4(IO_context *,Integer4 *var);
int CDECL __NAGf90_dusty_fmt_read_l(IO_context *,Logical *var);
int CDECL __NAGf90_dusty_fmt_read_l1(IO_context *,Logical1 *var);
int CDECL __NAGf90_dusty_fmt_read_l2(IO_context *,Logical2 *var);
int CDECL __NAGf90_dusty_fmt_read_l4(IO_context *,Logical4 *var);
#ifdef QUADR
int CDECL __NAGf90_dusty_fmt_read_q(IO_context *ioctx,QReal *var);
int CDECL __NAGf90_dusty_fmt_read_qc(IO_context *ioctx,QComplex *var);
#endif
int CDECL __NAGf90_dusty_fmt_read_r(IO_context *,Real *var);
int CDECL __NAGf90_dusty_fmt_write_c(IO_context *,Complex number);
int CDECL __NAGf90_dusty_fmt_write_d(IO_context *,Double number);
int CDECL __NAGf90_dusty_fmt_write_dc(IO_context *,DComplex number);
int CDECL __NAGf90_dusty_fmt_write_dd(IO_context *,DDReal number);
int CDECL __NAGf90_dusty_fmt_write_ddc(IO_context *,DDComplex number);
int CDECL __NAGf90_dusty_fmt_write_i(IO_context *,Integer number);
int CDECL __NAGf90_dusty_fmt_write_i1(IO_context *,Integer1 number);
int CDECL __NAGf90_dusty_fmt_write_i2(IO_context *,Integer2 number);
int CDECL __NAGf90_dusty_fmt_write_i4(IO_context *,Integer4 number);
int CDECL __NAGf90_dusty_fmt_write_l(IO_context *,Logical test);
int CDECL __NAGf90_dusty_fmt_write_l1(IO_context *,Logical1 test);
int CDECL __NAGf90_dusty_fmt_write_l2(IO_context *,Logical2 test);
int CDECL __NAGf90_dusty_fmt_write_l4(IO_context *,Logical4 test);
#ifdef QUADR
int CDECL __NAGf90_dusty_fmt_write_q(IO_context *ioctx,QReal number);
int CDECL __NAGf90_dusty_fmt_write_qc(IO_context *ioctx,QComplex number);
#endif
int CDECL __NAGf90_dusty_fmt_write_r(IO_context *,Real number);
void CDECL __NAGf90_end_inquire(IO_context *);
int CDECL __NAGf90_end_read_internal(IO_context *);
int CDECL __NAGf90_end_read_sequential(IO_context *);
int CDECL __NAGf90_end_read_unf(IO_context *);
int CDECL __NAGf90_end_read_direct_fmt(IO_context *);
int CDECL __NAGf90_end_read_direct_unf(IO_context *);
int CDECL __NAGf90_end_write_direct_fmt(IO_context *);
int CDECL __NAGf90_end_write_direct_unf(IO_context *);
int CDECL __NAGf90_end_write_ifile(IO_context *);
int CDECL __NAGf90_end_write_sequential(IO_context *);
int CDECL __NAGf90_end_write_unf(IO_context *);
int CDECL __NAGf90_endfilef(int,Char *,Chrlen,int);
int CDECL __NAGf90_endfilef_line(const char*,int,int,Char *,Chrlen,int);
int CDECL __NAGf90_establish_format(IO_context *,const Char *,Chrlen,int);
int CDECL __NAGf90_flushf(int,Char *,Chrlen,int);
int CDECL __NAGf90_flushf_line(const char*,int,int,Char *,Chrlen,int);
int CDECL __NAGf90_fmt_read_c(IO_context *,Complex *);
int CDECL __NAGf90_fmt_read_ch(IO_context *,Char *,Chrlen);
int CDECL __NAGf90_fmt_read_ch2j(IO_context *,Char2 *,Chrlen);
int CDECL __NAGf90_fmt_read_ch2u(IO_context *,Char2 *,Chrlen);
int CDECL __NAGf90_fmt_read_ch4(IO_context *,Char4 *,Chrlen);
int CDECL __NAGf90_fmt_read_d(IO_context *,Double *);
int CDECL __NAGf90_fmt_read_dc(IO_context *,DComplex *);
int CDECL __NAGf90_fmt_read_dd(IO_context *,DDReal *);
int CDECL __NAGf90_fmt_read_ddc(IO_context *,DDComplex *);
int CDECL __NAGf90_fmt_read_i(IO_context *,Integer *);
int CDECL __NAGf90_fmt_read_i1(IO_context *,Integer1 *);
int CDECL __NAGf90_fmt_read_i2(IO_context *,Integer2 *);
int CDECL __NAGf90_fmt_read_i4(IO_context *,Integer4 *);
int CDECL __NAGf90_fmt_read_l(IO_context *,Integer *);
int CDECL __NAGf90_fmt_read_l1(IO_context *,Logical1 *);
int CDECL __NAGf90_fmt_read_l2(IO_context *,Logical2 *);
int CDECL __NAGf90_fmt_read_l4(IO_context *,Logical4 *);
#ifdef QUADR
int CDECL __NAGf90_fmt_read_q(IO_context *ioctx,QReal *var);
int CDECL __NAGf90_fmt_read_qc(IO_context *ioctx,QComplex *var);
#endif
int CDECL __NAGf90_fmt_read_r(IO_context *,Real *);
int CDECL __NAGf90_fmt_write_c(IO_context *,Complex number);
int CDECL __NAGf90_fmt_write_ch(IO_context *,const Char *,Chrlen);
int CDECL __NAGf90_fmt_write_ch2j(IO_context *,const Char2 *,Chrlen);
int CDECL __NAGf90_fmt_write_ch2u(IO_context *,const Char2 *,Chrlen);
int CDECL __NAGf90_fmt_write_ch4(IO_context *,const Char4 *,Chrlen);
int CDECL __NAGf90_fmt_write_d(IO_context *,Double number);
int CDECL __NAGf90_fmt_write_dc(IO_context *,DComplex number);
int CDECL __NAGf90_fmt_write_dd(IO_context *,DDReal number);
int CDECL __NAGf90_fmt_write_ddc(IO_context *,DDComplex number);
int CDECL __NAGf90_fmt_write_i(IO_context *,Integer number);
int CDECL __NAGf90_fmt_write_i1(IO_context *,Integer1 number);
int CDECL __NAGf90_fmt_write_i2(IO_context *,Integer2 number);
int CDECL __NAGf90_fmt_write_i4(IO_context *,Integer4 number);
int CDECL __NAGf90_fmt_write_l(IO_context *,Logical number);
int CDECL __NAGf90_fmt_write_l1(IO_context *,Logical1 number);
int CDECL __NAGf90_fmt_write_l2(IO_context *,Logical2 number);
int CDECL __NAGf90_fmt_write_l4(IO_context *,Logical4 number);
#ifdef QUADR
int CDECL __NAGf90_fmt_write_q(IO_context *ioctx,QReal number);
int CDECL __NAGf90_fmt_write_qc(IO_context *ioctx,QComplex number);
#endif
int CDECL __NAGf90_fmt_write_r(IO_context *,Real number);
void CDECL __NAGf90_init_linput(IO_context *);
int CDECL __NAGf90_init_loutput(IO_context *);
void CDECL __NAGf90_inquire_access(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_action(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_asynchronous(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_blank(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_convert(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_decimal(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_delim(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_direct(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_encoding(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_exist(IO_context *,Logical *);
int CDECL __NAGf90_inquire_file(IO_context *,const Char *,Chrlen,Logical *,
                                Char *,Chrlen,Char *,Chrlen,int);
int CDECL __NAGf90_inquire_file_line(const char *,int,IO_context *,
                                     const Char *,Chrlen,Logical *,Char *,
                                     Chrlen,Char *,Chrlen,int);
void CDECL __NAGf90_inquire_form(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_formatted(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_name(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_name2j(IO_context *,Char2 *,Chrlen);
void CDECL __NAGf90_inquire_name2u(IO_context *,Char2 *,Chrlen);
void CDECL __NAGf90_inquire_name4(IO_context *,Char4 *,Chrlen);
void CDECL __NAGf90_inquire_named(IO_context *,Logical *);
void CDECL __NAGf90_inquire_nextrec(IO_context *,Integer4 *);
void CDECL __NAGf90_inquire_number(IO_context *,Integer4 *);
void CDECL __NAGf90_inquire_opened(IO_context *,Logical *);
void CDECL __NAGf90_inquire_pad(IO_context *,Char *,Chrlen);
int CDECL __NAGf90_inquire_pending(IO_context *,Logical *);
int CDECL __NAGf90_inquire_pending_id(IO_context *,Integer4,Logical *);
void CDECL __NAGf90_inquire_pos(IO_context *,Integer4 *);
void CDECL __NAGf90_inquire_position(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_read(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_readwrite(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_recl(IO_context *,Integer4 *);
void CDECL __NAGf90_inquire_round(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_sequential(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_sign(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_size(IO_context *,Integer4 *);
void CDECL __NAGf90_inquire_stream(IO_context *,Char *,Chrlen);
void CDECL __NAGf90_inquire_unformatted(IO_context *,Char *,Chrlen);
int CDECL __NAGf90_inquire_unit(IO_context *,Integer4,Char *,Chrlen,int);
int CDECL __NAGf90_inquire_unit_line(const char *,int,IO_context *,Integer4,
                                     Char *,Chrlen,int);
void CDECL __NAGf90_inquire_write(IO_context *,Char *,Chrlen);
int CDECL __NAGf90_lio_read_c(IO_context *,Complex *);
int CDECL __NAGf90_lio_read_ch(IO_context *,Char *,Chrlen);
int CDECL __NAGf90_lio_read_ch2j(IO_context *,Char2 *,Chrlen);
int CDECL __NAGf90_lio_read_ch2u(IO_context *,Char2 *,Chrlen);
int CDECL __NAGf90_lio_read_ch4(IO_context *,Char4 *,Chrlen);
int CDECL __NAGf90_lio_read_d(IO_context *,Double *);
int CDECL __NAGf90_lio_read_dc(IO_context *,DComplex *);
int CDECL __NAGf90_lio_read_dd(IO_context *,DDReal *);
int CDECL __NAGf90_lio_read_ddc(IO_context *,DDComplex *);
int CDECL __NAGf90_lio_read_i(IO_context *,Integer *);
int CDECL __NAGf90_lio_read_i1(IO_context *,Integer1 *);
int CDECL __NAGf90_lio_read_i2(IO_context *,Integer2 *);
int CDECL __NAGf90_lio_read_i4(IO_context *,Integer4 *);
int CDECL __NAGf90_lio_read_l(IO_context *,Logical *);
int CDECL __NAGf90_lio_read_l1(IO_context *,Logical1 *);
int CDECL __NAGf90_lio_read_l2(IO_context *,Logical2 *);
int CDECL __NAGf90_lio_read_l4(IO_context *,Logical4 *);
#ifdef QUADR
int CDECL __NAGf90_lio_read_q(IO_context *,QReal *);
int CDECL __NAGf90_lio_read_qc(IO_context *,QComplex *);
#endif
int CDECL __NAGf90_lio_read_r(IO_context *,Real *);
int CDECL __NAGf90_lio_write_c(IO_context *,Complex);
int CDECL __NAGf90_lio_write_ch(IO_context *,const Char *,Chrlen);
int CDECL __NAGf90_lio_write_ch2j(IO_context *,const Char2 *,Chrlen);
int CDECL __NAGf90_lio_write_ch2u(IO_context *,const Char2 *,Chrlen);
int CDECL __NAGf90_lio_write_ch4(IO_context *,const Char4 *,Chrlen);
int CDECL __NAGf90_lio_write_d(IO_context *,Double);
int CDECL __NAGf90_lio_write_dc(IO_context *,DComplex);
int CDECL __NAGf90_lio_write_dd(IO_context *,DDReal);
int CDECL __NAGf90_lio_write_ddc(IO_context *,DDComplex);
int CDECL __NAGf90_lio_write_i(IO_context *,Integer);
int CDECL __NAGf90_lio_write_i1(IO_context *,Integer1);
int CDECL __NAGf90_lio_write_i2(IO_context *,Integer2);
int CDECL __NAGf90_lio_write_i4(IO_context *,Integer4);
int CDECL __NAGf90_lio_write_l(IO_context *,Logical);
int CDECL __NAGf90_lio_write_l1(IO_context *,Logical1);
int CDECL __NAGf90_lio_write_l2(IO_context *,Logical2);
int CDECL __NAGf90_lio_write_l4(IO_context *,Logical4);
#ifdef QUADR
int CDECL __NAGf90_lio_write_q(IO_context *,QReal);
int CDECL __NAGf90_lio_write_qc(IO_context *,QComplex);
#endif
int CDECL __NAGf90_lio_write_r(IO_context *,Real);
typedef struct {
  Offset version;
  const Char *spec[15];
  Chrlen len[15];
} __NAGf90_open_specs;
int CDECL __NAGf90_openf2(int,Pointer,int,const Char*,Chrlen,
                          const __NAGf90_open_specs*,const Offset*,Char *,
                          Chrlen,int);
int CDECL __NAGf90_openf2g(const char *,int,int,Pointer,int,const Char*,Chrlen,
                           const __NAGf90_open_specs*,const Offset*,
                           Char *,Chrlen,int);
int CDECL __NAGf90_openf2x(int,Pointer,int,void *,Chrlen,int,
                           const __NAGf90_open_specs*,const Offset*,Char *,
                           Chrlen,int);
int CDECL __NAGf90_openf2gx(const char *,int,int,Pointer,int,void*,Chrlen,
                            int,const __NAGf90_open_specs*,const Offset*,
                            Char *,Chrlen,int);
int CDECL __NAGf90_read_direct_fmt(IO_context *,int,Offset,Char *,Chrlen,int);
int CDECL __NAGf90_read_direct_fmt_line(const char *,int,IO_context *,int,
                                        Offset,Char *,Chrlen,int);
int CDECL __NAGf90_read_direct_unf(IO_context *,int,Offset,Char *,Chrlen,int);
int CDECL __NAGf90_read_direct_unf_line(const char *,int,IO_context *,int,
                                        Offset,Char *,Chrlen,int);
int CDECL __NAGf90_read_internal_file(IO_context *,Internal_file *,Char *,
                                      Chrlen,int);
int CDECL __NAGf90_read_internal_file_line(const char *,int,IO_context *,
                                           Internal_file *,Char *,Chrlen,int);
int CDECL __NAGf90_read_sequential(IO_context*,int,Integer4*,Char*,Chrlen,int);
int CDECL __NAGf90_read_sequential_line(const char *,int,IO_context*,int,
                                        Integer4*,Char*,Chrlen,int);
int CDECL __NAGf90_read_unf(IO_context *,int,Integer4 *,Char *,Chrlen,int);
int CDECL __NAGf90_read_unf_line(const char *,int,IO_context *,int,Integer4 *,
                                 Char *,Chrlen,int);
int CDECL __NAGf90_read_unf2_block(IO_context *,Pointer,Offset,const BS_Info *);
int CDECL __NAGf90_read_unf2_dv(IO_context *,int,Pointer,Triplet *,Offset,
                                const BS_Info *);
int CDECL __NAGf90_readu_optf2(int,Char *,Chrlen,int,Pointer,Offset,
                               const BS_Info *);
int CDECL __NAGf90_readu_optf2g(const char *,int,int,Char *,Chrlen,int,Pointer,
                                Offset,const BS_Info *);
int CDECL __NAGf90_rewindf(int,Char *,Chrlen,int);
int CDECL __NAGf90_rewindf_line(const char *,int,int,Char *,Chrlen,int);
void CDECL __NAGf90_scalar_ifilex(Internal_file *,void *,Offset,int);
int CDECL __NAGf90_set_ioctx(IO_context*,const Char*,Chrlen,const Char*,Chrlen,
                             const Char*,Chrlen,const Char*,Chrlen,const Char*,
                             Chrlen,const Char *,Chrlen);
int CDECL __NAGf90_set_ioctx_advance(IO_context *,const Char *,Chrlen);
void CDECL __NAGf90_set_ioctx_nonadvancing(IO_context *);
int CDECL __NAGf90_set_ioctx_size(IO_context *,Integer4 *);
int CDECL __NAGf90_waitf(int,int*,Char *,Chrlen,int);
int CDECL __NAGf90_waitf_line(const char*,int,int,int*,Char *,Chrlen,int);
int CDECL __NAGf90_write_ifile(IO_context *,Internal_file *,Char *,Chrlen,int);
int CDECL __NAGf90_write_ifile_line(const char *,int,IO_context *,
                                    Internal_file *,Char *,Chrlen,int);
int CDECL __NAGf90_write_sequential(IO_context *,int,Integer4 *,Char *,Chrlen,
                                    int);
int CDECL __NAGf90_write_sequential_line(const char *,int,IO_context *,int,
                                         Integer4 *,Char *,Chrlen,int);
int CDECL __NAGf90_writeu_optf2(int,Char *,Chrlen,int,Pointer,Offset,
                                const BS_Info *);
int CDECL __NAGf90_writeu_optf2g(const char *,int,int,Char *,Chrlen,int,
                                 Pointer,Offset,const BS_Info *);
int CDECL __NAGf90_write_unf(IO_context *,int,Integer4 *,Char *,Chrlen,int);
int CDECL __NAGf90_write_unf_line(const char *,int,IO_context *,int,Integer4 *,
                                  Char *,Chrlen,int);
int CDECL __NAGf90_write_unf2_block(IO_context *,Pointer,Offset,
                                    const BS_Info *);
int CDECL __NAGf90_write_unf2_dv(IO_context*,int,Pointer,const Triplet*,Offset,
                                 const BS_Info *);
int CDECL __NAGf90_write_direct_fmt(IO_context *,int,Offset,Char *,Chrlen,int);
int CDECL __NAGf90_write_direct_fmt_line(const char *,int,IO_context *,int,
                                         Offset,Char *,Chrlen,int);
int CDECL __NAGf90_write_direct_unf(IO_context *,int,Offset,Char *,Chrlen,int);
int CDECL __NAGf90_write_direct_unf_line(const char *,int,IO_context *,int,
                                         Offset,Char *,Chrlen,int);

typedef void (*__NAGf90_final_subroutine)();
typedef struct {
  __NAGf90_final_subroutine final[22];
  void *reserved1,*reserved2;
  int reserved3,reserved4;
} __NAGf90_final_subroutines;
struct __NAGf90_signature {
  int depth;
  Offset size;
  char *typename;
  void (*copyproc)();
  void (*finalise)();
  void (*initialise)();
  void (*reserved1)();
  void *reserved2;
  __NAGf90_final_subroutines *ufinal;
};
extern struct __NAGf90_signature __NAGf90_I1_HEADER,__NAGf90_I2_HEADER,
  __NAGf90_I3_HEADER,__NAGf90_I4_HEADER,__NAGf90_L1_HEADER,__NAGf90_L2_HEADER,
  __NAGf90_L3_HEADER,__NAGf90_L4_HEADER,__NAGf90_R1_HEADER,__NAGf90_R2_HEADER,
  __NAGf90_R3_HEADER,__NAGf90_C1_HEADER,__NAGf90_C2_HEADER,__NAGf90_C3_HEADER,
  __NAGf90_CH_HEADER,__NAGf90_CH2_HEADER,__NAGf90_CH3_HEADER,
  __NAGf90_CH4_HEADER,__NAGf90_DD_HEADER,__NAGf90_DDC_HEADER;
void CDECL __NAGf90_tls_init(volatile int *);
int CDECL __NAGf90_tls_iget(int);
void CDECL __NAGf90_tls_iset(int,int);
void * CDECL __NAGf90_tls_pget(int idx);
void CDECL __NAGf90_tls_pset(int idx,void *val);

typedef void (CDECL *__NAGf90_OpenMP_construct_procptr)(void *);
void CDECL __NAGf90_OpenMP_atomic_add_i1(Integer1*,Integer1);
void CDECL __NAGf90_OpenMP_atomic_add_i2(Integer2*,Integer2);
void CDECL __NAGf90_OpenMP_atomic_add_i3(Integer3*,Integer3);
void CDECL __NAGf90_OpenMP_atomic_add_i4(Integer4*,Integer4);
void CDECL __NAGf90_OpenMP_atomic_and_l1(Logical1*,Logical1);
void CDECL __NAGf90_OpenMP_atomic_and_l2(Logical2*,Logical2);
void CDECL __NAGf90_OpenMP_atomic_and_l3(Logical*,Logical);
void CDECL __NAGf90_OpenMP_atomic_and_l4(Logical4*,Logical4);
void CDECL __NAGf90_OpenMP_atomic_lock(void);
void CDECL __NAGf90_OpenMP_atomic_neqv_l1(Logical1*,Logical1);
void CDECL __NAGf90_OpenMP_atomic_neqv_l2(Logical2*,Logical2);
void CDECL __NAGf90_OpenMP_atomic_neqv_l3(Logical*,Logical);
void CDECL __NAGf90_OpenMP_atomic_neqv_l4(Logical4*,Logical4);
void CDECL __NAGf90_OpenMP_atomic_or_l1(Logical1*,Logical1);
void CDECL __NAGf90_OpenMP_atomic_or_l2(Logical2*,Logical2);
void CDECL __NAGf90_OpenMP_atomic_or_l3(Logical*,Logical);
void CDECL __NAGf90_OpenMP_atomic_or_l4(Logical4*,Logical4);
void CDECL __NAGf90_OpenMP_atomic_unlock(void);
void CDECL __NAGf90_OpenMP_barrier(void);
void __NAGf90_OpenMP_critical_begin(void *);
void __NAGf90_OpenMP_critical_end(void *);
void * CDECL __NAGf90_OpenMP_do_1(void *,INT64,INT64,INT64,int,INT64*,Offset,
                                  Pointer *);
void CDECL __NAGf90_OpenMP_do_end(void *,int);
int CDECL __NAGf90_OpenMP_do_loop_1(void *,INT64*,INT64*);
void CDECL __NAGf90_OpenMP_flush(void);
void CDECL __NAGf90_OpenMP_gbarrier(int,const char *);
void CDECL __NAGf90_OpenMP_info(int *,int *);
int CDECL __NAGf90_OpenMP_ismaster(void);
int CDECL __NAGf90_OpenMP_parallel(__NAGf90_OpenMP_construct_procptr,
                                   const char *,int,int,void *,int,INT64 *,
                                   Offset);
int CDECL __NAGf90_OpenMP_section_next2(void *);
int CDECL __NAGf90_OpenMP_section_wnext(void *);
void * CDECL __NAGf90_OpenMP_sections(void *,int,Offset,Pointer*);
void CDECL __NAGf90_OpenMP_sections_endpara(void *);
void CDECL __NAGf90_OpenMP_sections_endwait(void *);
void * CDECL __NAGf90_OpenMP_sections_wait(void *,int,Offset,Pointer*);
int CDECL __NAGf90_OpenMP_single(void *);
int CDECL __NAGf90_OpenMP_single_wait(void);
void CDECL __NAGf90_OpenMP_single_endwait(void);

#ifndef NOVALUE
extern float __NAGf90_fvalue[];
extern double __NAGf90_dvalue[];
extern DDReal __NAGf90_ddvalue[];
#ifdef QUADR
extern QReal __NAGf90_qvalue[];
#endif
#endif
